package edu.cmu.sei.pacc.absdom;

import junit.framework.TestCase;

import edu.toronto.cs.algebra.TwoValAlgebra;

import edu.toronto.cs.expr.*;
import edu.toronto.cs.mvset.*;

import static edu.toronto.cs.expr.BoolOp.*;

public class MvSetAbstractDomainTestCase extends TestCase
{
  
  
  public void testSimplify ()
  {
    // -- an expression factory, there is typically a single instance
    // -- of an expression factory per application
    ExprFactory eFac = new ExprFactoryImpl ();
    
    
    // -- an abstract domain is created with an MvSet factory to
    // -- construct abstract elements, and an expression factory to
    // -- construct concrete expressions. When used as a simplifier,
    // -- the abstract domain should use it's own special MvSet
    // -- factory and not necessarily the one used for analysis. That
    // -- is why no local reference to the MvSet factory is retained
    // -- in this example.
    MvSetAbstractDomain aDom = 
      new MvSetAbstractDomain 
      (new MDDMvSetFactory (new TwoValAlgebra (), 10), eFac);
    

    // -- create a random expression
    Expr e = land (eFac.var ("x"), lnot (eFac.var ("y")));

    // -- turn it into a redundant one
    e = or (e, e);

    System.out.println ("Original Expression:");
    System.out.println (e);


    // -- simplification is done by doing 'aDom.alpha' to build an
    // -- mvset for the expression, and 'aDom.gamma' to concretize it
    // -- back into the expression. 
    System.out.println ("Simplified Expression:");
    System.out.println (aDom.gamma (aDom.alpha (e)));

    // -- Note that this class may also be used to represent and
    // -- manipulate expressions as mvsets during various stages of
    // -- program abstraction. Many operations can be more efficient
    // -- on mvsets since they are always reduced.
  }

  public static void main (String[] args)
  {
    new MvSetAbstractDomainTestCase ().testSimplify ();
  }
  
  
}
