package edu.toronto.cs.clang;

import edu.toronto.cs.expr.*;
import edu.toronto.cs.tp.SymbolTable;
import edu.toronto.cs.util.IdentityHashSet;

import static edu.toronto.cs.expr.CTypedDeclOp.*;
import static edu.toronto.cs.expr.CLocOp.*;

import java.util.*;

/**
 * CSymbolTable.java
 *
 * A symbol table for C programs.
 *
 * This is not a SymbolTable, since it does not map from variable Exprs to
 * type Exprs. However, a SymbolTable should be produced from it.
 */
public class CSymbolTable
{
  /**
   * typeTable may be redundant since a CVarDecl points to its CType.
   */
  Map<Integer, Expr> typeTable; // tuid -> CType
  Map<Integer, Expr> declTable; // uid -> CVarDecl

  private CSymbolTable () { }

  public CSymbolTable
  (Map<Integer, Expr> typeTable, Map<Integer, Expr> declTable)
  {
    this.typeTable = typeTable;
    this.declTable = declTable;
  }

  /**
   * The type of a CDecl is available via the decl itself; for a loc without a
   * meaningful uid, the corresponding decl can be looked up by name--this
   * lookup is not implemented yet.
   */
  public Expr typeOf (Expr e) throws Exception
  {
    if (isCTypedDecl (e))
      return declType (e);

    if (isCLoc (e))
      // fails for loc without a proper uid
      return declType (declTable.get (CLocOp.uid (e)));

    throw new Exception ("Cannot get type of " + e.op ());
  }

  public Expr getDecl (int uid)
  {
    return declTable.get (uid);
  }

  public final Map<Integer, Expr> getDeclTable ()
  {
    return declTable;
  }

  public final Set<Expr> decls ()
  {
    return new IdentityHashSet (declTable.values ());
  }

  public String toString ()
  {
    return "types: " + typeTable.toString ()
      + ", decls: " + declTable.toString ();
  }
}
