package edu.toronto.cs.expr;

import java.util.List;


/**
 * A base class for implementing the {@code Expr} interface. This
 * class provides implementation of all the methods that are
 * independent of any particular implementation of the {@code Expr}
 * interface.
 *
 * @author <a href="mailto:arie@cs.toronto.edu">Arie Gurfinkel</a>
 * @version 1.0
 */
public abstract class AbstractExpr implements Expr
{
  public Expr nullaryApply ()
  {
    return naryApply (Expr.EMPTY_EXPR_ARRAY);
  }
  
  public Expr unaryApply (Expr expr)
  {
    return naryApply (new Expr[] {expr});
  }

  public Expr binApply (Expr expr1, Expr expr2)
  {
    return naryApply (new Expr[] {expr1, expr2});
  }
  public Expr naryApply (List expr)
  {
    return naryApply ((Expr[]) expr.toArray (new Expr [expr.size ()]));
  }

  public boolean contains (Expr e)
  {
    if (this == e)
      return true;

    for (int i = 0; i < arity (); i++)  
      if (arg (i) != null && arg (i).contains (e))
        return true;

    return false;  
  }

  
}
