package edu.toronto.cs.expr;

/**
 * A C declaration with a context.  Each declaration has a unique
 * decalaration id (uid) which uniquelly defines it. Declarations may
 * have optional attributes such as a name and a context.
 *
 */
public abstract class CDeclOp extends AbstractOperator 
{

  int uid;
  Expr context;
  
  
  public CDeclOp (String name, int arity, int _uid, Expr _context)
  {
    super (name, arity);
    uid = _uid;
    context = _context;
  }


  public static boolean isDecl (Expr e)
  {
    return (e.op () instanceof CDeclOp);
  }

  public static Expr declContext (Expr e)
  {
    return ((CDeclOp)e.op ()).context;
  }

  public static int declUID (Expr e)
  {
    return ((CDeclOp)e.op ()).uid;
  }

  public boolean equals (Object o)
  {
    // -- two declarations are equal if they have the same uid. Other
    // -- attributes are optional and are irrelevant for equality
    return o != null && (o instanceof CDeclOp) && (uid == ((CDeclOp)o).uid);
  }
  
  public int hashCode ()
  {
    // -- a declaration is completelly defined by its unique id
    return uid;
  }
  
}
