package edu.toronto.cs.expr;

/** 
 * Non-function decalarations: primitives, pointers, structs, function
 * pointers.
 * 
 * @author Kelvin Ku (kelvin@cs.toronto.edu)
 * @version 
 */
public class CILDeclarationOp extends NamedOp
{
  // variable/record declaration
  // 0 list (type specifiers and qualifiers, can be an entire struct tree)
  // 1 0 CILDeclarationPointerOp (pointer group)
  // 1 1 var (identifier)
  // 1 2 list (parameter list of function pointer declaration)
  // 1 3 list of array dimension descriptions, one per each dimension
  // Comment, for now we assume that only one of 1 2 or 1 3 is non-null
  // XXX not sure what the above line means; can't they both be null for a
  // non-array, non-fptr declaration?

  public static final CILDeclarationOp DECL =
    new CILDeclarationOp ("declaration", -1);

  // -- constructor

  private CILDeclarationOp (String name, int arity)
  {
    super (name, arity);
  }

  public static boolean isNamedDeclaration (Expr e)
  {
    assert isDecl (e);

    return e.arity () > 1 && e.arg (1) != null;
  }

  public static String getDeclName (Expr e)
  {
    return getDeclNameExpr (e).op ().name ();
  }

  public static Expr getDeclNameExpr (Expr e)
  {
    if (isNamedDeclaration (e))
      return e.arg (1).arg (1);

    throw new RuntimeException ("malformed decl: " + e);
  }

  public static Expr getBaseType (Expr e)
  {
    // XXX assume the base type is the last thing in the list
    // this'd be wrong if we were taking size into account, and 
    // we had "short int x", since it'd give us "int" instead
    // of "short"

    //System.out.println("Getting base type of " + e);
    Expr tmp = e.arg (0);
    return (tmp. arg (tmp.arity () - 1));
  }

  public static int getNumPtrs (Expr e)
  {
    if (e.arg (1).arg (0) != null)
      return CILDeclarationPointerOp.getNumPtrs (e.arg (1).arg (0));

    return 0;
  }

  /**
   * Returns {@code true} if {@code e} is an array declaration
   *
   * @param e an <code>Expr</code> value
   * @return a <code>boolean</code> value
   */
  public static boolean isArrayDecl (Expr e)
  {
    // -- check that e is a decl
    assert isDecl (e) : e + " is not a declaration";
    //System.out.println("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%");
    //System.out.println("e.arg(1) " + e.arg(1) + "%%%" + e.arg(1).arg(3));
    
    return e.arity () > 1 
	 && e.arg (1).arity () > 3 
	 && e.arg (1).arg (3) != null;
  }

  /**
   * Returns a list describing the array dimensions if {@code e} is an
   * array declaration, and returns {@code null} if {@code e} is not.
   *
   * @param e an <code>Expr</code> value
   * @return an <code>Expr</code> value
   */
  public static Expr arrayDim (Expr e)
  {
    if (!isArrayDecl (e)) return null;
    return e.arg (1).arg (3);
  }
  
  public static boolean isDecl (Expr e)
  {
    return e.op () == DECL;
  }
}
