package edu.toronto.cs.expr;

public class CILIndirectionOp extends NamedOp
{
  public static final CILIndirectionOp
    DEREF   = new CILIndirectionOp ("*", 1),
    DOT     = new CILIndirectionOp (".", 2),
    ARROW   = new CILIndirectionOp ("->", 2),
    ADDR_OF = new CILIndirectionOp ("&", 1);

  public static boolean isDeref (Expr e)
  {
    return e.op () == DEREF;
  }

  public static boolean isAddrOf (Expr e)
  {
    return e.op () == ADDR_OF;
  }

  public static boolean isDot (Expr e)
  {
    return e.op () == DOT;
  }

  public static boolean isArrow (Expr e)
  {
    return e.op () == ARROW;
  }

  private CILIndirectionOp (String name, int arity)
  {
    super (name, arity);
  }

  // Creation methods

  /**
   * Returns a dereference {@code *p}
   *
   * @param e an <code>Expr</code> value
   * @return an <code>Expr</code> value
   */
  public static final Expr deref (Expr e)
  {
    return e.getFactory ().op (DEREF).unaryApply (e);
  }

  /**
   * Returns a field access {@code foo.bar}
   *
   * @param e1 an <code>Expr</code> value
   * @param e2 an <code>Expr</code> value
   * @return an <code>Expr</code> value
   */
  public static final Expr dot (Expr e1, Expr e2)
  {
    return e1.getFactory ().op (DOT).binApply (e1, e2);
  }

  /**
   * Returns a field access {@code foo.bar}
   *
   * @param e1 an <code>Expr</code> value
   * @param n an <code>int</code> value
   * @return an <code>Expr</code> value
   */
  public static final Expr dot (Expr e1, int n)
  {
    return dot (e1, e1.getFactory ().intExpr (n));
  }

  /**
   * Returns a field access from a pointer {@code foo->bar}
   *
   * @param e1 an <code>Expr</code> value
   * @param e2 an <code>Expr</code> value
   * @return an <code>Expr</code> value
   */
  public static final Expr arrow (Expr e1, Expr e2)
  {
    return e1.getFactory ().op (ARROW).binApply (e1, e2);
  }

  /**
   * Returns a address of a variable {@code &x}
   *
   * @param e an <code>Expr</code> value
   * @return an <code>Expr</code> value
   */
  public static final Expr addrOf (Expr e)
  {
    return e.getFactory ().op (ADDR_OF).unaryApply (e);
  }

}
