package edu.toronto.cs.expr;

/**
 * Represents a C statement: an expression which can appear at the top level
 * of a basic block statement list.
 */
public class CStmtOp extends NamedOp implements NonCanonOp
{
  Integer lineNum;

  public static final CStmtOp MODIFY_EXPR = new CStmtOp ("=", 2);
  public static final CExprOp COND_EXPR = new CExprOp ("condExpr", 3); 
  public static final CStmtOp RETURN = new CStmtOp ("return", -1);
  public static final CStmtOp LABEL_EXPR = new CStmtOp ("label", 1);
  public static final CStmtOp GOTO = new CStmtOp ("goto", 1);

  public CStmtOp (String name, int arity)
  {
    super (name, arity);
  }

  public static boolean isModifyExpr (Expr e)
  {
    return e.op () == MODIFY_EXPR;
  }

  public static boolean isCondExpr (Expr e)
  {
    return e.op () == COND_EXPR;
  }

  public static final Expr condExprCond (Expr e)
  {
    return e.arg (0);
  }
  public static final Expr condExprThen (Expr e)
  {
    return e.arg (1);
  }
  
  public static final Expr condExprElse (Expr e)
  {
    return e.arg (2);
  }
  

  public static boolean isReturn (Expr e)
  {
    return e.op () == RETURN;
  }

  public static boolean isLabelExpr (Expr e)
  {
    return e.op () == LABEL_EXPR;
  }

  public static boolean isGoto (Expr e)
  {
    return e.op () == GOTO;
  }

  public static final Expr gotoDest (Expr e)
  {
    return e.arg (0);
  }
  

  public static Expr labelExprDecl (Expr e)
  {
    return e.arg (0);
  }

  public static Expr modifyExpr (Expr lhs, Expr rhs)
  {
    return lhs.getFactory ().op (MODIFY_EXPR).binApply (lhs, rhs);
  }
}
