package edu.toronto.cs.expr;

/**
 * A switch statement.
 *
 * Children
 * switchCond	(SWITCH_COND)
 * switchBody	(SWITCH_BODY)
 */
public class CSwitchExprOp extends CStmtOp
{
  public static final CSwitchExprOp SWITCH_EXPR =
    new CSwitchExprOp ("switchExpr", -1);

  public static final CSwitchExprOp SWITCH_COND =
    new CSwitchExprOp ("switchCond", 1);

  public static final CSwitchExprOp SWITCH_BODY =
    new CSwitchExprOp ("switchBody", -1);

  public static final CSwitchExprOp CASE_LABEL_EXPR =
    new CSwitchExprOp ("caseLabelExpr", -1);

  public static final CSwitchExprOp DEFAULT_CASE =
    new CSwitchExprOp ("defaultCase", 0);

  public CSwitchExprOp (String name, int arity)
  {
    super (name, arity);
  }

  public static boolean isSwitchExpr (Expr e)
  {
    return (e.op () == SWITCH_EXPR);
  }

  public static boolean isDefaultCase (Expr e)
  {
    return (e.op () == DEFAULT_CASE);
  }

  public static Expr switchCond (Expr e)
  {
    return e.arg (0);
  }

  public static Expr switchBody (Expr e)
  {
    return e.arg (1);
  }

  public static int numCases (Expr e)
  {
    return switchBody (e).arity () / 2;
  }

  public static Expr caseLabel (Expr e, int i)
  {
    return switchBody (e).arg (2*i);
  }

  public static Expr caseGoto (Expr e, int i)
  {
    return switchBody (e).arg (2*i+1);
  }

  /**
   * Returns the low element of the range for case i; the low element is
   * DEFAULT_CASE if case i is the "default" case.
   */
  public static Expr caseLow (Expr e, int i)
  {
    return caseLabel (e, i).arg (0);
  }

  /**
   * Returns the high element of the range for case i; returns null if case i
   * is the "default" case or if the range is a single value.
   */
  public static Expr caseHigh (Expr e, int i)
  {
    if (caseLabel (e, i).arity () > 1)
      return caseLabel (e, i).arg (1);

    return null;
  }
}
