package edu.toronto.cs.expr;

/**
 * A rewriter for disjunctions.
 *
 * Predicates are strengthened using big disjunctions of cubes, which
 * are written in disjunctive normal form (DNF). {@code
 * DisjunctionRewriter} is intended to be used on such expressions: it
 * writes each disjunct on its on line. For large disjunctions of
 * cubes, this makes the expression much more readable.
*/
public class DisjunctionRewriter extends IdentityRewriter
{

  public DisjunctionRewriter ()
  {
  }
  
  public Expr rewrite (Expr e)
  {
    // Transform children first (postorder)
    e = super.rewrite (e);

    return (BoolOp.isOr (e)) ?
      e.getFactory ().op (BoolOp.ORN).binApply (e.arg (0), e.arg (1)) 
      : e;
  }


  public static void main (String args[])
  {
    DisjunctionRewriter d = new DisjunctionRewriter ();
    ExprFactory fac = new ExprFactoryImpl ();
    Expr x = fac.var ("x");
    Expr y = fac.var ("y");

    Expr e = BoolOp.or (BoolOp.and (x,y), BoolOp.and(x, BoolOp.not (y)));
    Expr e2 = BoolOp.and (e, BoolOp.or (e,x));

    System.out.println (e);
    System.out.println (d.rewrite (e));

    System.out.println (e2);
    System.out.println (d.rewrite (e2));
  }
}