package edu.toronto.cs.expr;

/**
 * Various functions for pretty-printing expressions.
 *
 */
public class Fmt
{

  public static String disjuncts (Expr e, String indent) {
	/* Special case for choice op. */
	if (ChoiceOp.isChoice (e)) {
	  return (indent + "ch (" + disjuncts (e.arg (0), indent) 
			  + "\n , \n" 
			  + disjuncts (e.arg (1), indent) + ")");
	}
	if (BoolOp.isOr (e)) {
	  return ""
            + disjuncts (e.arg (0), indent) 
            + " \\/ \n  " + indent
            +  disjuncts (e.arg (1), indent);
	} else {
	  return e.toString ();
	}
  }

  /**
   * Pretty-prints expressions that are disjunctions at the top level.
   * If the expression is a choice op at the top level, we print 
   * the top-level disjuncts of each choice.
   *
   * @param e and @code{Expr}
   * @return a @code{String} showing the top-level disjuncts, one per line
   *
   */
  public static String disjuncts (Expr e)
  {
    return disjuncts (e, "");
  }


  private static String tree (Expr e, int spaces)
  {
	StringBuffer sb = new StringBuffer (spaces * 2 + 1);
	for (int i = 0; i < spaces * 2; i++)
	  sb.append (' ');
	sb.append ((char) 0);
	String spacer = new String (sb);

    if (e.arity () == 0)
      return spacer + e.toString () + "\n";
	
	String ret = spacer + e.op ().name () + "\n";
    for (Object arg : e.args ())
      ret += tree ((Expr) arg, spaces+1);
    return ret;
  }


  /** 
   *
   * Prints the entire expression tree of an @code{Expr}
   *
   */
  public static String tree (Expr e)
  {
	return tree (e, 0);
  }
}
