package edu.toronto.cs.expr;

import java.util.*;

/** 
 * A base rewriter which traverses a given expression and leaves it unmodified.
 * <p>
 * Extend this with a rewriter whose rewrite method first calls {@code e =
 * super.rewrite (e)} on its argument {@code e} to rewrite its children, then
 * locally transforms {@code e}; this amounts to a post-order (i.e., children
 * first, bottom up) rewriting.
 * 
 * @author Kelvin Ku (kelvin@cs.toronto.edu)
 * @version 
 */
public class IdentityRewriter implements ExprRewriter
{
  public IdentityRewriter ()
  {
  }

  public Expr rewrite (Expr e)
  {
    if (e.arity () == 0)
      return e;
    List<Expr> rArgs = new ArrayList<Expr> (e.args ().size ());
    for (Object arg : e.args ())
      rArgs.add (rewrite ((Expr) arg));
    return e.getFactory ().op (e.op ()).naryApply (rArgs);
  }
}

