package edu.toronto.cs.expr;

public class NumericOp extends NamedOp
{
  public static final NumericOp PLUS = new NumericOp ("+", -1);
  public static final NumericOp MINUS = new NumericOp ("-", 2);
  public static final NumericOp UN_MINUS = new NumericOp ("-", 1);
  public static final NumericOp MULT = new NumericOp ("*", 2);
  public static final NumericOp DIV = new NumericOp ("/", 2);
  public static final NumericOp MOD = new NumericOp ("mod", 2);
  public static final NumericOp ABS = new NumericOp ("abs", 1);
  
  private NumericOp (String name, int arity)
  {
    super (name, arity);
  }

  public static final boolean isPlus (Expr e)
  {
    return e.op () == PLUS;
  }

  public static final boolean isMinus (Expr e)
  {
    return e.op () == MINUS;
  }

  // Creation Methods

  /**
   * Returns the sum of its arguments.
   *
   * @param e1 an <code>Expr</code> value
   * @param e2 an <code>Expr</code> value
   * @return an <code>Expr</code> value
   */
  public static final Expr plus (Expr e1, Expr e2)
  {
    return e1.getFactory ().op (PLUS).binApply (e1, e2);
  }

  /**
   * Returns the sum of its arguments.
   *
   * @param e1 an <code>Expr</code> value
   * @param n and integer value
   * @return an <code>Expr</code> value
   */
  public static final Expr plus (Expr e1, int n)
  {
    return plus (e1, e1.getFactory ().intExpr (n));
  }

  /**
   * Returns the sum of its arguments.
   *
   * @param expr a non-empty array of <code>Expr</code> values
   * @return an <code>Expr</code> value
   */
  public static final Expr plus (Expr [] expr)
  {
    assert expr.length >= 0 : "Cannot create sum of an empty array!";
    return expr[0].getFactory ().op (PLUS).naryApply (expr);
  }

  /**
   * Returns the difference of its arguments {@code e1 - e2}.
   *
   * @param e1 an <code>Expr</code> value
   * @param e2 an <code>Expr</code> value
   * @return an <code>Expr</code> value
   */
  public static final Expr minus (Expr e1, Expr e2)
  {
    return e1.getFactory ().op (MINUS).binApply (e1, e2);
  }

  /**
   * Returns the difference of its arguments {@code e1 - n}.
   *
   * @param e1 an <code>Expr</code> value
   * @param n an integer
   * @return an <code>Expr</code> value
   */
  public static final Expr minus (Expr e1, int n)
  {
    return minus (e1, e1.getFactory ().intExpr (n));
  }

  /**
   * Returns the negation of its argument {@code -e}.
   *
   * @param e an <code>Expr</code> value
   * @return an <code>Expr</code> value
   */
  public static final Expr un_minus (Expr e)
  {
    return e.getFactory ().op (UN_MINUS).unaryApply (e);
  }

  /**
   * Returns the product of its arguments {@code e1 * e2}.
   *
   * @param e1 an <code>Expr</code> value
   * @param e2 an <code>Expr</code> value
   * @return an <code>Expr</code> value
   */
  public static final Expr mult (Expr e1, Expr e2)
  {
    return e1.getFactory ().op (MULT).binApply (e1, e2);
  }

  /**
   * Returns the product of its arguments {@code e1 * n}.
   *
   * @param e1 an <code>Expr</code> value
   * @param n an integer value
   * @return an <code>Expr</code> value
   */
  public static final Expr mult (Expr e1, int n)
  {
    return mult (e1, e1.getFactory ().intExpr (n));
  }

  /**
   * Returns the a ratio of its arguments {@code e1 / e2}.
   *
   * @param e1 an <code>Expr</code> value
   * @param e2 an <code>Expr</code> value
   * @return an <code>Expr</code> value
   */
  public static final Expr DIV (Expr e1, Expr e2)
  {
    return e1.getFactory ().op (DIV).binApply (e1, e2);
  }

  /**
   * Returns the a ratio of its arguments {@code e1 / n}.
   *
   * @param e1 an <code>Expr</code> value
   * @param n an integer value
   * @return an <code>Expr</code> value
   */
  public static final Expr DIV (Expr e1, int n)
  {
    return DIV (e1, e1.getFactory ().intExpr (n));
  }

  /**
   * Returns the modulus of its arguments {@code e1 % e2}.
   *
   * @param e1 an <code>Expr</code> value
   * @param e2 an <code>Expr</code> value
   * @return an <code>Expr</code> value
   */
  public static final Expr mod (Expr e1, Expr e2)
  {
    return e1.getFactory ().op (MOD).binApply (e1, e2);
  }

  /**
   * Returns the modulus of its arguments {@code e1 % n}.
   *
   * @param e1 an <code>Expr</code> value
   * @param n an integer value
   * @return an <code>Expr</code> value
   */
  public static final Expr mod (Expr e1, int n)
  {
    return mod (e1, e1.getFactory ().intExpr (n));
  }

  /**
   * Returns the modulus of its arguments {@code e1 % n}.
   *
   * @param e an <code>abs</code> value
   * @return an <code>Expr</code> value
   */
  public static final Expr abs (Expr e)
  {
    return e.getFactory ().op (ABS).unaryApply (e);
  }
}
