package edu.toronto.cs.expr;

import edu.toronto.cs.util.Primes;

public class RationalOp extends NullaryOperator
{
  int n;
  int d;

  public RationalOp (int _n, int _d)
  {
    n = _n;
    d = _d;
  }
  
  public String name ()
  {
    return d == 1 ? String.valueOf (n) : n + "/" + d;
  }
  
  public boolean equals (Object v)
  {
    if (this == v) return true;
   
    return 
      v != null && 
      v instanceof RationalOp && 
      ((RationalOp)v).getN () == this.n &&
      ((RationalOp)v).getD () == this.d;
  }

  public int hashCode ()
  {
    long hashCode = Primes.getPrime (0) * n + Primes.getPrime (1) * d;
    return (int) (hashCode >>> 32);
  }

  public int getN ()
  {
    return n;
  }
  public int getD ()
  {
    return d;
  }


  /**
   * A helper method that extracts the numerator of a rational
   * expression.
   *
   * @param expr an <code>Expr</code> whose operator is assumed to be
   * {@code RationalOp}
   * @return an <code>int</code> value
   */
  public static int getN (Expr expr)
  {
    return ((RationalOp)expr.op ()).getN ();
  }

  /**
   * A helper method that extracts the denumerator of a rational
   * expression.
   *
   * @param expr an <code>Expr</code> value
   * @return an <code>int</code> value
   */
  public static int getD (Expr expr)
  {
    return ((RationalOp)expr.op ()).getD ();
  }


  /**
   * Returns {@code true} if {@code expr} is an integer
   *
   * @param expr an <code>Expr</code> value
   * @return a <code>boolean</code> value
   */
  public static boolean isInt (Expr expr)
  {
    return expr.op ().getClass () == RationalOp.class &&
      getD (expr) == 1;
  }
  
    
}
