package edu.toronto.cs.expr;


/**
 * Simple types including integers, reals, void, and dummy.
 *
 * @author <a href="mailto:arie@kanga.fm.sandbox">Arie Gurfinkel</a>
 * @version 1.0
 */
public class SimpleTypeOp extends NamedOp
{
  /**
   * A type of integers.
   *
   */
  public static final SimpleTypeOp INT = new SimpleTypeOp ("INT");

  /**
   * A type of chars.
   *
   */
  public static final SimpleTypeOp CHAR = new SimpleTypeOp ("CHAR");

  /**
   * A type of reals.
   *
   */
  public static final SimpleTypeOp REAL = new SimpleTypeOp ("REAL");

  /**
   * A unit void type.
   *
   */
  public static final SimpleTypeOp VOID = new SimpleTypeOp ("VOID");


  /**
   * A dummy type to be used to represent a type that cannot be
   * repressented.
   *
   */
  public static final SimpleTypeOp DUMMY = new SimpleTypeOp ("DUMMY");

  /** 
   * An address type. 
   */
  public static final SimpleTypeOp ADDR = new SimpleTypeOp ("ADDR");

  /** 
   * Returns true if {@code e} is an integer type. 
   * 
   * @param e 
   * @return 
   */
  public static boolean isIntT (Expr e)
  {
    return e.op () == INT;
  }

  /** 
   * Returns true if {@code e} is a character type. 
   * 
   * @param e 
   * @return 
   */
  public static boolean isCharT (Expr e)
  {
    return e.op () == CHAR;
  }

  /** 
   * Returns true iff {@code e} is an address type. 
   * 
   * @param e 
   * @return 
   */
  public static boolean isAddrT (Expr e)
  {
    return e.op () == ADDR;
  }
  
  public SimpleTypeOp (String name)
  {
    super (name, 0);
  }
  
}
