package edu.toronto.cs.expr;

import static edu.toronto.cs.expr.BoolOp.*;
import static edu.toronto.cs.expr.ComparisonOp.*;
import static edu.toronto.cs.expr.CILIndirectionOp.*;
import static edu.toronto.cs.expr.NumericOp.*;
import static edu.toronto.cs.expr.RationalOp.*;

/** 
 * A rewriter that simplifies expressions by applying various equivalence
 * rules, including those of Boolean logic, arithmetic, and C.
 * 
 * @author Kelvin Ku (kelvin@cs.toronto.edu)
 * @version 
 */
public class SimplifyingRewriter extends IdentityRewriter
{
  public Expr rewrite (Expr e)
  {
    e = super.rewrite (e);

    if (isNot (e))
      return lnot (e.arg (0));
    else if (isAnd (e))
      return land (e.arg (0), e.arg (1));
    else if (isOr (e))
      return lor (e.arg (0), e.arg (1));
    else if (e.op () == EQ)
      return lequal (e.arg (0), e.arg (1));
    else if (isGt (e))
      return lgt (e.arg (0), e.arg (1));
    else if (isLt (e))
      return llt (e.arg (0), e.arg (1));
    else if (isDeref (e) && isAddrOf (e.arg (0)))
      return (e.arg (0).arg (0));
    else if (isAddrOf (e) && isDeref (e.arg (0)))
      return (e.arg (0).arg (0));
    else if (isMinus (e) // x - 0 = x
             && e.arg (1) == e.getFactory ().intExpr (0))
      return e.arg (0);
    else if (isMinus (e) // x - x = 0
             && e.arg (0) == e.arg (1))
      return e.getFactory ().intExpr (0);
    else if (isMinus (e) // difference of two constants can be simplified
             && isInt (e.arg (0)) && isInt (e.arg (1)))
      return e.getFactory ().intExpr (getN (e.arg (0)) - getN (e.arg (1)));
    else if (isPlus (e)  // x + 0 = x
             && e.arg (1) == e.getFactory ().intExpr (0))
      return e.arg (0);
    else if (isPlus (e)  // 0 + x = x
             && e.arg (0) == e.getFactory ().intExpr (0))
      return e.arg (1);
    else if (isPlus (e) // sum of two constants can be simplified
             && isInt (e.arg (0)) && isInt (e.arg (1)))
      return e.getFactory ().intExpr (getN (e.arg (0)) + getN (e.arg (1)));
    else
      return e;
  }

  public static Expr simplify (Expr e)
  {
    return (new SimplifyingRewriter ()).rewrite (e);
  }
}


