package edu.toronto.cs.expr;

import java.util.List;

/** 
 * Defines a tuple.
 *
 * ((Expr x) (Expr y))
 * 
 * @author <a href="mailto:tomhart@cs.toronto.edu">Tom Hart</a>
 * @version 
 */
public class TupleOp extends NamedOp
{
  public static final TupleOp PAIR =
    new TupleOp (",", 2);

  public static final TupleOp TUPLE =
    new TupleOp (",", -1);

  /** 
   * Ugly, but needed. This is a pair that should be printed without
   * enclosing brackets.
   */
  public static final TupleOp PAIR_NB =
    new TupleOp (",", 2);

  public TupleOp ()
  {
    super ("tuple", -1);
  }

  public TupleOp (String name)
  {
    super (name, -1);
  }

  public TupleOp (String name, int arity)
  {
    super (name, arity);
  }

  public static boolean isTuple (Expr e)
  {
    return (e.op () instanceof TupleOp);
  }

  /** 
   * Returns true iff the given expression is a pair (ie. 2-tuple). 
   * 
   * @param e 
   * @return 
   */
  public static boolean isPair (Expr e)
  {
    return e.op () == PAIR;
  }

  /**
   * Returns a 2-tuple {@code (Expr, Expr)}
   *
   * @param e1 an <code>Expr</code> value
   * @param e2 an <code>Expr</code> value
   * @return an <code>Expr</code> value
   */
  public static final Expr tuple (Expr e1, Expr e2)
  {
    return e1.getFactory ().op (PAIR).binApply (e1, e2);
  }

  /**
   * Returns a 2-tuple {@code (Expr, Expr)}
   *
   * @param e1 an <code>Expr</code> value
   * @param n an integer value
   * @return an <code>Expr</code> value
   */
  public static final Expr tuple (Expr e1, int n)
  {
    return tuple(e1, e1.getFactory ().intExpr (n));
  }

  /**
   * Returns a 2-tuple {@code (Expr, Expr)}, no brackets when printing.
   *
   * @param e1 an <code>Expr</code> value
   * @param e2 an <code>Expr</code> value
   * @return an <code>Expr</code> value
   */
  public static final Expr tupleNB (Expr e1, Expr e2)
  {
    return e1.getFactory ().op (PAIR_NB).binApply (e1, e2);
  }

  /**
   * Returns an n-ary tuple with children {@code elements}.
   */
  public static final Expr tuple (List<Expr> elements)
  {
    return
    elements.get (0).getFactory ().op (new TupleOp ()).naryApply (elements);
  }
}

