package edu.toronto.cs.expr;

import java.util.Set;

public class VariableOp extends NullaryOperator
{
  String name;
  
  public VariableOp (String _name)
  {
    name = _name;
  }
  
  public String name ()
  {
    return name;
  }
  
  public int hashCode ()
  {
    return name.hashCode ();
  }
  
  public boolean equals (Object v)
  {
    if (this == v) return true;
    if (v != null && v instanceof VariableOp) 
      return ((VariableOp)v).name ().equals (name ());
    return false;
  }

  public static final boolean isVar (Expr e)
  {
    return (e.op () instanceof VariableOp);
  }

  public static final String varName (Expr expr)
  {
    return ((VariableOp)expr.op ()).name ();
  }

  public static final boolean isVariable (Expr expr)
  {
    return isVar (expr);
  }
  
  /** 
   * Collects all variables in an expression.
   *
   * @param e an expression whose variables are sought
   * @param s a set into which variables are returned
   * @return the set {@code s}
   * @throws NullPointerException if either {@code e} or {@code s} are null
   */
  public static final Set<Expr> vars (Expr e, Set<Expr> s)
  {
    if (isVar (e))
      s.add (e);
    else
      for (int i = 0; i < e.arity (); i++)
	vars (e.arg (i), s);

    return s;
  }
  
}
