package edu.toronto.cs.proof;

import java.util.*;
import edu.toronto.cs.util.StringUtil;
import edu.toronto.cs.openjgraph.*;
import edu.toronto.cs.openjgraph.OpenJGraph.*;
import edu.toronto.cs.algebra.*;
import edu.toronto.cs.modelchecker.*;
import edu.toronto.cs.ctl.*;
import edu.toronto.cs.mvset.MvSet;



public class ProofToOpenJ extends AbstractProofVisitor 
{  
  
  OpenJGraph graph = new OpenJGraph ();
  Map seen = new HashMap ();

  static String makeLabel (String stateName,
			   MvSet state)
  {
    StringBuffer sb = new StringBuffer (stateName + ": ");
    
    StatePresenter sp = ProofStepFactory.getStructure ().getStatePresenter ();
    
    CTLNode[] lbl = sp.toCTL (state);

    for (int i = 0; i < lbl.length; i++)
      {
	String lft = lbl[i].getLeft ().toString ();
	String rt = lbl[i].getRight ().toString ();
	sb.append("\n" + lft + "=" + rt);   
	//Had to use "/n" instead of "//n" unlike Grappa and daVinci
	sb.append("\n" + lft + "=" + rt);
      }
    return sb.toString();
  }
  
  static String makeLabel(String stateName, 
			  MvSet pred,
			  MvSet curr)
  {
    StringBuffer sb = new StringBuffer (stateName+": ");
    StatePresenter sp = ProofStepFactory.getStructure ().getStatePresenter ();
    CTLNode[] oldlbl = sp.toCTL (pred);
    CTLNode[] newlbl = sp.toCTL (curr);
    
    int j = 0;
    for (int i = 0; i < oldlbl.length; i++)
      {
	// only add those which are different
	if (!newlbl[i].getRight ().equals (oldlbl[i].getRight ()))
	  {
	    sb.append("\n" + newlbl[i].getLeft ().toString () + "=" +
		      newlbl[i].getRight ().toString ());
	    //Had to use "/n" instead of "//n" unlike Grappa and daVinci
	    j++;
	  }
      }
    return sb.toString();
  }

  OpenJNode makeProofNode (String lab)
  {
    return graph.node().label(lab);
    /*.
      shape("box").
      fontStyle("italic").
      fontFamily("times").     
      color("lightblue").      
      style("filled").
      label(lab);
    */
  }
  
  OpenJNode makeStateNode(String lab)
  {
    return graph.node().label(lab);
    /*.
      shape ("box").
      fontStyle ("bold").
      fontFamily ("times").
      label (lab);
    */
  }
  
  public static OpenJGraph toOpenJ (ProofStep ps)
  {
    ProofToOpenJ ptdv = new ProofToOpenJ ();
    OpenJNode rootState = 
      ptdv.makeStateNode (makeLabel (ps.getStateName (),
				     ps.getState ()));

//       ptdv.makeStateNode (makeLabel (ps.getStateName (),
// 				     ps.getStateAsArray ()));
    
    String lbl = new String("[" + ps.getConsequent ().toString() + "]=(" +
			    ps.getValue() + ")");
    OpenJNode rootProof = ptdv.makeProofNode (lbl);
    



    ptdv.graph.setRoot (rootState);

    rootState.edge (rootProof);
    
    ptdv.seen.put (ps.getStateName (), rootState);
    
    ps.accept (ptdv,
	       new OpenJNodePair (rootProof, rootState));
    return ptdv.graph;
  }
  


  public Object visitGeneric (ProofStep ps, Object info)
  {
    OpenJNodePair nodes = (OpenJNodePair)info;
    
    for (Iterator it = ps.getAntecedents ().iterator (); it.hasNext (); )
      {
	ProofStep pst = (ProofStep)it.next ();
	String lbl = new String("[" + pst.getConsequent () + "]=(" +
				pst.getValue() + ")");
	
	OpenJNode child = makeProofNode (lbl);
	
	nodes.fst.edge (child).color ("lightpurple");
	pst.accept (this, new OpenJNodePair(child, nodes.snd));
      }
    return info;
  }
  
    public Object visitEXStep(ProofStep ps, Object info)
    {
      OpenJNodePair parentNodes = (OpenJNodePair) info;
    
      OpenJNode[] kids = new OpenJNode[ps.getAntecedents().size()];
    
      for (Iterator it = ps.getAntecedents().iterator();
  	 it.hasNext();
  	 )
        {
  	ProofStep ant = (ProofStep) it.next();
  	String sname = ant.getStateName();

	// create witness state node
  	OpenJNode fnode = makeStateNode(makeLabel(sname,
						  ps.getState (),
						  ant.getState ()));

// 						  ps.getStateAsArray(),
// 						  ant.getStateAsArray()));
	
	// create proof antecedent node
	OpenJNode pnode = makeProofNode(new String("["+ant.
						  getConsequent().
						  toString()+"]=("+
						  ant.getValue()+")"));
	
	// link EX node to the witness state with transition value
  	MvSetModelChecker mcc = ProofStepFactory.getMC();
  	XKripkeStructure struct = ProofStepFactory.getStructure();
//   	AlgebraValue tv =
// 	  mcc.getTrans().toMvSet ().and(ps.getState()).
//   	  and(ant.getState().renameArgs(mcc.getPrime())).
//   	  existAbstract(struct.getPrimeCube().
//   			and(struct.getUnPrimeCube())).getValue();
	  AlgebraValue tv = 
	    mcc.getTrans ().
	    fwdImage (ps.getState ()).and (ant.getState ()).
	    existAbstract (struct.getUnPrimeCube ()).getValue ();



	parentNodes.fst.edge(fnode).fixLabel().label(tv.toString()); //XXX do labels later;
	
	parentNodes.snd.edge(fnode).fixLabel().label(tv.toString()); //.lineType("double");  XXX do labels later

	
	// and link the proof antecedent to the witness state.
	fnode.edge(pnode);
  	ant.accept(this, new OpenJNodePair(pnode, fnode));
	
        }
      return null;
    }
}
