package edu.toronto.cs.tp;

import java.util.List;
import java.util.Iterator;

import edu.toronto.cs.expr.*;


/**
 * Convinience class for implementing {@code
 * edu.toronto.cs.tp.ITheoremProver} itnerface.
 *
 * @author <a href="mailto:arie@kanga.fm.sandbox">Arie Gurfinkel</a>
 * @version 1.0
 */
public abstract class AbstractTheoremProver implements ITheoremProver
{

  ExprFactory exprFactory;
  
  /**
   * Initializes {@code exprFactory}
   *
   * @param _exprFactory an <code>ExprFactory</code> value
   */
  public  AbstractTheoremProver (ExprFactory _exprFactory)
  {
    exprFactory = _exprFactory;
  }
  

  public ExprFactory exprFactory ()
  {
    return exprFactory;
  }
  
  public boolean quickQuery (Expr expr)
  {
    int level = stackLevel ();
    push ();
    boolean result = query (expr);
    popto (level);
    return result;
  }

  /*
   * @deprecated Use {@link #checkTruth (Expr)} with a choice expression.
   */
  public Boolean checkTruth (Expr tExpr, Expr fExpr)
  {
    int level = stackLevel ();
    
    push ();
    boolean result = query (tExpr);
    if (result) return Boolean.TRUE;
    
    popto (level);
    push ();
    result = query (fExpr);
    if (result) return Boolean.FALSE;
    return null;
  }

  public Boolean quickCheckTruth (Expr tExpr, Expr fExpr)
  {
    int level = stackLevel ();
    push ();
    Boolean result = checkTruth (tExpr, fExpr);
    popto (level);
    return result;
  }

  public Boolean checkTruth (Expr expr)
  {
    Expr tExpr = ChoiceOp.lgetT (expr);
    Expr fExpr = ChoiceOp.lgetF (expr);

    int level = stackLevel ();
    
    push ();
    boolean result = query (tExpr);
    if (result) return Boolean.TRUE;
    
    popto (level);
    push ();
    result = query (fExpr);
    if (result) return Boolean.FALSE;
    return null;
  }
  
  public Boolean quickCheckTruth (Expr expr)
  {
    int level = stackLevel ();
    push ();
    Boolean result = checkTruth (expr);
    popto (level);
    return result;
  }

  public boolean implies (Expr e1, Expr e2)
  {
    return quickQuery (exprFactory ().op (BoolOp.IMPL).binApply (e1, e2));
  }
  

  public void assertFormula (List<Expr> l)
  {
    for (Expr e : l)
      assertFormula (e);
  }
  
}

