package edu.toronto.cs.tp;

import edu.toronto.cs.expr.*;
import edu.toronto.cs.tp.*;
import static edu.toronto.cs.expr.ComparisonOp.*;
import static edu.toronto.cs.expr.NumericOp.*;

import java.util.*;
import junit.framework.TestCase;

/**
 * A collection of tests for CLCVCLTheoremProver
 *
 * @author <a href="mailto:anton@age.cs">Anton Selyshchev</a>
 * @version 1.0
 */
public class CLCVCLTheoremProverTestCase extends TestCase {
  public void testMethod(){

    ExprFactory fac = new ExprFactoryImpl ();
    CLCVCLTheoremProver tp = new CLCVCLTheoremProver (fac);

    System.out.println("=================================PART 1====");

    Expr x = fac.var ("x");
    Expr y = fac.var ("y");
    tp.declare (x, fac.op (SimpleTypeOp.INT));
    tp.declare (y, fac.op (SimpleTypeOp.INT));

    Expr f1 = fac.op (ComparisonOp.GT).binApply (x, fac.intExpr (7));
    Expr f2 = fac.op (ComparisonOp.GT).binApply (x, fac.intExpr (0));
    Expr f3 = fac.op (ComparisonOp.GT).binApply (y, x);

    tp.assertFormula (f1);
    tp.assertFormula (f2);
    tp.assertFormula (f3);

    List<Expr> expectedOutput =new ArrayList();
    expectedOutput.add (f1);
    	 
     
    System.out.println (tp.query (fac.op (ComparisonOp.GT).binApply (x, fac.intExpr (1))));
        
    assertEquals(expectedOutput, tp.getAssumptionsUsed());
    System.out.println (tp.query (fac.op (ComparisonOp.GT).binApply (x, fac.intExpr (-1))));
    assertEquals(expectedOutput, tp.getAssumptionsUsed());
  
    expectedOutput =new ArrayList();
    
    expectedOutput.add (f1);
    expectedOutput.add (f3);
    

    System.out.println (tp.query (fac.op (ComparisonOp.GT).binApply (y, fac.intExpr (5))));
    assertEquals(expectedOutput, tp.getAssumptionsUsed());


    System.out.println("=================================PART 3====");
    //-- check if push, pop work properly 

    tp.assertFormula (fac.op (ComparisonOp.GT).binApply (y, fac.intExpr (10)));
    tp.push();
    tp.assertFormula (fac.op (ComparisonOp.GT).binApply (x, fac.intExpr (20)));

    System.out.println (tp.query (fac.op (ComparisonOp.GT).binApply (x, fac.intExpr (1))));

    tp.getAssumptionsUsed();

    assertEquals(tp.stackLevel(), 1);

    tp.pop();

    assertEquals(tp.stackLevel(), 0);

    tp.push();
    tp.push();
    tp.push();
    tp.push();

    assertEquals(tp.stackLevel(), 4);




    System.out.println("=================================PART 4====");
    tp.push();

    assertEquals(tp.stackLevel(), 5);

    tp.popto (2);

    assertEquals(tp.stackLevel(), 2);



 
  }

  public void testPopTo ()
  {
    int sl;
    ExprFactory fac = new ExprFactoryImpl ();
    ITheoremProver tp = new CLCVCLTheoremProver (fac);
    boolean q;

    tp.push ();

    sl = tp.stackLevel ();
    tp.popto (sl);
    tp.popto (sl);
  }
}
