package edu.toronto.cs.tp;

import java.io.*;
import java.util.Arrays;


/**
 * Test case to see how to use CVCL process.
 *
 * @author <a href="mailto:arie@kanga.fm.sandbox">Arie Gurfinkel</a>
 * @version 1.0
 */
public class CVCLTester
{
  static String cvclCmd;
  static Process cvclProcess;
  static char[] cvclPrompt = {'C', 'V', 'C', '>', ' '};
  
  
  public static void main (String[] args)
  {
    if (args.length != 1)
      {
	System.out.println ("usage: java " + CVCLTester.class + 
			    " path_to_cvcl_executable");
	return;
      }
    
    cvclCmd = args [0];

    // -- we want to start cvcl in interactive mode
    String[] cmdString = new String [] {cvclCmd, "+int"};
    

    // -- create a process
    try {
      System.out.println ("Starting CVCL with " + Arrays.asList (cmdString));
      cvclProcess = Runtime.getRuntime ().exec (cmdString);
    } catch (Exception ex) {
      System.out.println ("Failed to start " + cvclCmd + " because of " + ex);
      return;
    }
    
    BufferedReader userIn = new BufferedReader 
      (new InputStreamReader (System.in));
    
    BufferedReader stdIn;
    Writer stdOut;
    
    stdIn = new BufferedReader 
      (new InputStreamReader (cvclProcess.getInputStream ()));

    stdOut = new OutputStreamWriter (cvclProcess.getOutputStream ());



    try {
      while (true)
	{
	  System.out.println ("From CVCL:");
	  System.out.println (readUntilPrompt (stdIn, "CVC> "));
	  System.out.println ("End of CVCL output");

	  System.out.println ("Enter commmand: ");
	  System.out.print ("HERE> ");
	  String userCmd = userIn.readLine ();
	  if (userCmd.equalsIgnoreCase ("quit"))
	    {
	      System.out.println ("You want to quit, bye");
	      cvclProcess.destroy ();
	      return;
	    }
	  else
	    System.out.println ("Sending your command to CVCL...");
	
	  stdOut.write (userCmd);
	  stdOut.append ('\n');
	  stdOut.flush ();
	}
    }
    catch (IOException ex)
      {
	System.out.println ("Sorry, we are having I/O problems: " + ex);
	ex.printStackTrace ();
      }
    

  }


  
  /**
   * Reads input until prompt.
   *
   * @param in the input to read
   * @param prompt the prompt string
   * @return what was read
   */
  static String readUntilPrompt (Reader in, String prompt) throws IOException
  {
    StringBuffer sb = new StringBuffer ();
    String result;
    
    while (true)
      {
	// -- wait for the first character
	sb.append ((char) in.read ());
	// -- read everything there is
	while (in.ready ())
	  sb.append ((char) in.read ());
	result = sb.toString ();
    
	// -- check that we saw the prompt again, if so, return
	if (result.endsWith (prompt))
	  return result;
      }
  }
  
  /**
   * Read input until exhaustion, and waits for at least one input symbol.
   *
   * @param in a <code>Reader</code> value
   * @return a <code>String</code> value
   * @exception IOException if an error occurs
   */
  static String readOneFully (Reader in) throws IOException
  {
    StringBuffer sb = new StringBuffer ();
    // -- wait for the first character
    sb.append ((char) in.read ());
    
    // -- now read everything there is
    while (in.ready ())
      sb.append ((char) in.read ());
    
    return sb.toString ();
  }
  

  static boolean waitForInput (BufferedReader in, char[] expected)
    throws IOException
  {
    // -- nothing to wait for
    if (expected == null || expected.length == 0) return true;

    // -- what is the next character we expect to see
    int seen = 0;
    
    // -- keep looking until we see everything
    while (seen < expected.length)
      {
	int ch = in.read ();
	if (ch == -1 || ((char)ch) != expected [seen])
	  // -- failed
	  return false;
	seen++;
      }
    return true;
  }
  
}
