package edu.toronto.cs.yasm.abspatterns;

import edu.toronto.cs.expr.*;

public class AbstractionPatternEntry {

  int numVars;
  int type; /* 0 -> ex, 1 -> str */
  Expr bound; 
  Expr array;
  Expr leader;
  int varDirection; /* 0 -> up, 1 -> down, 2 -> unknown */
  int leaderDirection;

  public String toString ()
  {
    String base = "<numVars='" + numVars + "'>"
      + "<type='" + (type==0 ? "ex" : "str") + "'>";
    String boundString = "";
    if (bound != null) boundString = "<bound='" + bound.toString () + "'>";
    String arrayString = "";
    if (array != null) arrayString = "<array='" + array.toString () + "'>";
    String leaderString = "";
    if (leader != null) arrayString = "<leader='" + leader.toString () + "'>";

    return base + boundString + arrayString + leaderString;
  }

  /* Accessors */
  public int getNumVars ()         
  { 
    if (numVars != 1 && numVars != 2)
      throw new RuntimeException ("numVars invalid");
    return numVars; 
  }

  public int getType ()            
  { 
    if (type != 0 && type != 1)
      throw new RuntimeException ("type invalid");
    return type; 
  }

  public Expr getBound ()        
  { 
    if (bound == null)
      throw new RuntimeException ("bound invalid");
    return bound; 
  }

  public Expr getArray ()        
  {
    if (array == null)
      throw new RuntimeException ("array invalid");
    return array; 
  }

  public Expr getLeader ()       
  {
    if (leader == null)
      throw new RuntimeException ("leader invalid");
    return leader; 
  }

  /* Don't bother throwing runtime exceptions here. */
  public int getVarDirection ()    { return varDirection; }
  public int getLeaderDirection () { return leaderDirection; }

  /* Initialize every field that some abstraction may not use to 
   * a default value.
   */
  private void baseInit (int _numVars, int _type) {
    numVars = _numVars;
    type = _type;
    bound = null;
    array = null;
    leader = null;
    varDirection = 2;
    leaderDirection = 2;
  }

  /* Initializer for pattern 1 ex or 1 str */
  public AbstractionPatternEntry (int _numVars,
                                  int _type,
                                  Expr _ba) {
    baseInit (_numVars, _type);
    if (_type == 0) bound = _ba;
    else if (_type == 1) array = _ba;
    else { throw new RuntimeException ("Invalid abstraction type"); }
  }

  /* Initializer for pattern 2 ex */
  public AbstractionPatternEntry (int _numVars,
                                  int _type,
                                  Expr _bound,
                                  Expr _leader,
                                  int _varDirection,
                                  int _leaderDirection) {
    baseInit (_numVars, _type);
    bound = _bound;
    leader = _leader;
    varDirection = _varDirection;
    leaderDirection = _leaderDirection;
  }

  /* Initializer for pattern 2 str */
  public AbstractionPatternEntry (int _numVars,
                                  int _type,
                                  Expr _array,
                                  Expr _leader) {
    baseInit (_numVars, _type);
    array = _array;
    leader = _leader;
  }
}
