package edu.toronto.cs.yasm.abspatterns;

import java.util.Set;
import java.io.*;

public class AbstractionPatternFileParser {
  /* The names of fields that we can see in a .abs file. 
   * Not currently used anywhere, but could be used for parsing. 
   */
  String [] fieldNames = 
  { "function", "loop", "var", "numvars", "type", "bound", "array",
    "varDirection", "leader", "leaderDirection" };

  BufferedReader in;
  String curLine;

  String function;
  String loop;
  String var;
  int numVars;
  int type;
  String bound;
  String array;
  String leader;
  int varDirection;
  int leaderDirection;

  /* Accessors */
  public String getFunction ()     { return function; }
  public String getLoop ()         { return loop; }
  public String getVar ()          { return var; }
  public int getNumVars ()         { return numVars; }
  public int getType ()            { return type; }
  public String getBound ()        { return bound; }
  public String getArray ()        { return array; }
  public String getLeader ()       { return leader; }
  public int getVarDirection ()    { return varDirection; }
  public int getLeaderDirection () { return leaderDirection; }


  private void reInitFields () {
    function = null;
    loop = null;
    var = null;
    numVars = -1;
    type = -1;
    bound = null;
    array = null;
    leader = null;
    varDirection = -1;
    leaderDirection = -1;
  }

  public AbstractionPatternFileParser (String absFileName) {
    try {
      in = new BufferedReader(new FileReader(absFileName));
    } catch (java.io.FileNotFoundException ex) {
      System.err.println (absFileName + " Not Found");
      ex.printStackTrace (System.err);
    }

  }

  public int readLine ()
  {
    try {
      curLine = in.readLine ();
      if (curLine == null) return -1;
      reInitFields ();
      processLine (curLine);
    } catch (IOException ex) {
      System.err.println (ex);
      ex.printStackTrace (System.err);
    }
    return 0;
  }

  private void processLine (String line) {
    int endOfNextPair, endOfPrevPair;
    String currentPair;

    for (endOfPrevPair = 0; ; endOfPrevPair = endOfNextPair) {
      /* Get the end of the next pair, breaking if we're done. */
      endOfNextPair = getEndOfNextPair (line.substring (endOfPrevPair));
      if (endOfNextPair == -1) break; 
      endOfNextPair += endOfPrevPair;

      /* Process the current pair. */
      currentPair = line.substring (endOfPrevPair, endOfNextPair);
      processPair (currentPair);
    }
  }

  private void processPair (String pair) {
    /* pair is of the form <.*='.*'> */
    int equalsPos = pair.indexOf ('=');
    String key = pair.substring (1,equalsPos);
    String value = pair.substring (equalsPos+2, pair.length () - 2);
    //System.out.println ("key = " + key + "\tvalue = " + value);
    if (key.equals ("function")) {
      function = value;
    } else if (key.equals ("loop")) {
      loop = value;
    } else if (key.equals ("var")) {
      var = value;
    } else if (key.equals ("numvars")) {
      numVars = Integer.parseInt (value);
    } else if (key.equals ("type")) {
      if (value.equals ("ex")) type = 0;
      else type = 1;
    } else if (key.equals ("bound")) {
      bound = value;
    } else if (key.equals ("array")) {
      array = value;
    } else if (key.equals ("leader")) {
      leader = value;
    } else if (key.equals ("varDirection")) {
      if (value.equals ("up")) varDirection = 0;
      else if (value.equals ("down")) varDirection = 1;
    } else if (key.equals ("leaderDirection")) {
      if (value.equals ("up")) leaderDirection = 0;
      else if (value.equals ("down")) leaderDirection = 1;
    }
  }

  private int getEndOfNextPair (String line) {
    String baseEnder = "'><";
    int baseEnd = line.indexOf (baseEnder);
    if (baseEnd == -1) {
      if (line.endsWith (">")) return line.length ();
      else return -1;
    }
    return baseEnd + 2;
  }

}

