package edu.toronto.cs.yasm.abspatterns;

import edu.toronto.cs.yasm.pprogram.*;
import edu.toronto.cs.expr.*;

import static edu.toronto.cs.expr.BoolOp.*;

import java.util.List;
import java.util.ArrayList;

/**
 * Collects the conditionals in a loop. Invoke using
 * @code{getConditionals(WhilePStmt)}.
 *
 */
public class ConditionCollectorVisitor extends BasePProgramVisitor 
{

  WhilePStmt myLoop;
  List<Expr> conditions;

  /** Only public entry point. */
  public static List<Expr> getConditionals (WhilePStmt loop) 
  {
    ConditionCollectorVisitor v = new ConditionCollectorVisitor (loop);
    loop.accept (v, null);
    return v.getConditions ();
  }

  private ConditionCollectorVisitor (WhilePStmt _myLoop)
  {
    myLoop = _myLoop;
    conditions = new ArrayList<Expr> ();
  }

  private List<Expr> getConditions ()
  {
    return conditions;
  }

  /* descend into ifs */
  public Object visitIf (IfPStmt s, Object o)
  {
    Expr cond = s.getCond ().getOrigCond ();
    if (!isTrue (cond) && !isFalse (cond))
      conditions.add (cond);

    if (s.getThenGoto () != null) {
      PStmt ts = s.getThenStmt ();
      while (ts != null) {
        ts.accept (this, o);
        ts = ts.getNext();
      }
    }

    if (s.getElseGoto () != null) {
      PStmt ts = s.getElseStmt ();
      while (ts != null) {
        ts.accept (this, o);
        ts = ts.getNext();
      }
    }

    return visitPStmt (s, o);
  }

  
}
