package edu.toronto.cs.yasm.abspatterns;

import edu.toronto.cs.yasm.pprogram.*;

/**
 * Class for determining if a statement is in the scope of a
 * conditional statement. You cannot instantiate this class. To use
 * it, call:
 *
 *    HasStmtVisitor.hasStmt (haystack, needle);
 *
 */
public class HasStmtVisitor extends BasePProgramVisitor 
{

  /** Only public entry point. */
  public static boolean hasStmt (IfPStmt haystack, PStmt needle) 
  {
    HasStmtVisitor v = new HasStmtVisitor (needle);
    haystack.accept (v, null);
    return v.needleFound ();
  }

  PStmt needle;
  boolean found;
  private HasStmtVisitor (PStmt _needle)
  {
    needle = _needle;
    found = false;
  }

  private boolean needleFound ()
  {
    return found;
  }

  /* descend into ifs */
  public Object visitIf (IfPStmt s, Object o)
  {

    if (s.getThenGoto () != null) {
      PStmt ts = s.getThenStmt ();
      while (ts != null) {
        ts.accept (this, o);
        ts = ts.getNext();
      }
    }

    if (s.getElseGoto () != null) {
      PStmt ts = s.getElseStmt ();
      while (ts != null) {
        ts.accept (this, o);
        ts = ts.getNext();
      }
    }

    return visitPStmt (s, o);
  }

  /* if this statement is the one we're looking for, set found to true */
  public Object visitPStmt (PStmt s, Object o)
  {
    if (!found)
      if (s == needle) 
        found = true;
    return o;
  }
  
  
}
