package edu.toronto.cs.yasm.abspatterns;

import edu.toronto.cs.expr.*;
import edu.toronto.cs.yasm.pprogram.*;
import java.util.List;
import java.util.ArrayList;

import static edu.toronto.cs.expr.VariableOp.*;

public class NullposCollectorVisitor extends BasePProgramVisitor 
{
  List<Expr> nullposes;

  public NullposCollectorVisitor ()
  {
    nullposes = new ArrayList<Expr> ();
  }

  public List<Expr> getNullposes ()
  {
    return nullposes;
  }

  public Expr getArray (Expr A_nullpos)
  {
    assert nullposes.contains (A_nullpos) : 
      "Not a nullpos: " + A_nullpos;

    String npName = varName (A_nullpos);
    String arrayName = npName.substring (0, npName.indexOf ("_nullpos"));
    System.out.println ("array of " + A_nullpos + " is " + arrayName);
    return A_nullpos.getFactory ().var (arrayName);
  }

  public Object visitPProgram (PProgram p, Object o)
  {
    if (!p.getGlobalDecls ().isEmpty ()) {
      for (Object pd : p.getGlobalDecls ().values ())
        ((PDecl)pd).accept (this, o);
    }
    
    for (Object pfd : p.getFunctionDefs ().values ())
      ((PFunctionDef)pfd).accept (this, o);

    return o;
  }

  public Object visitFunctionDef (PFunctionDef fdef, Object o)
  {
    if (!fdef.getLocalDecls ().isEmpty ()) {
      for (Object pd : fdef.getLocalDecls ().values ())
        ((PDecl)pd).accept (this, o);
    }

    return o;
  }


  public Object visitDecl (PDecl d, Object o)
  {
    Expr sourceExpr;
    
    sourceExpr = d.getSourceExpr ();

    String name = CILDeclarationOp.getDeclName (sourceExpr);

    //System.out.println(" name is " + name);

    if (name.endsWith ("_nullpos"))
      nullposes.add (sourceExpr.getFactory ().var (name));

    return o;

  }

}
