package edu.toronto.cs.yasm.abspatterns;

import edu.toronto.cs.expr.*;
import edu.toronto.cs.yasm.pprogram.*;
import java.util.List;
import java.util.ArrayList;

import static edu.toronto.cs.yasm.abspatterns.HasStmtVisitor.*;

/**
 * In the long term, return loops for which (1) needle is in scope,
 * and (2) cond is an exit predicate. For now, just return all loops
 * for which needle is in scope.
 *
 */
public class UnrolledLoopCollectorVisitor extends BasePProgramVisitor 
{
  Expr cond;
  PrllAsmtPStmt needle;
  List<IfPStmt> loops;

  /** Only public entry point. */
  public static List<IfPStmt> getLoops (Expr _cond, PrllAsmtPStmt _needle)
  {
    UnrolledLoopCollectorVisitor v = 
      new UnrolledLoopCollectorVisitor (_cond, _needle);
    PFunctionDef func = _needle.getParent ();
    func.accept (v, null);

    return v.loops;
  }

  private UnrolledLoopCollectorVisitor (Expr _cond, PrllAsmtPStmt _needle)
  {
    cond = _cond;
    needle = _needle;
    loops = new ArrayList<IfPStmt> ();
  }

  /* descend into ifs */
  public Object visitIf (IfPStmt s, Object o)
  {

    if (s.getThenGoto () != null) {
      PStmt ts = s.getThenStmt ();
      while (ts != null) {
        ts.accept (this, o);
        ts = ts.getNext();
      }
    }

    if (s.getElseGoto () != null) {
      PStmt ts = s.getElseStmt ();
      while (ts != null) {
        ts.accept (this, o);
        ts = ts.getNext();
      }
    }

    return visitPStmt (s, o);
  }

  public Object visitWhile (WhilePStmt s, Object o)
  {
    if (hasStmt (s, needle)) loops.add (s);
    return visitIf (s, o);
  }

  public Object visitFunctionDef (PFunctionDef fdef, Object o)
  {
    if ( fdef.getHead () != null) {	
      PStmt s = fdef.getEntryPoint (); 
      
      while (s != null) {
        s.accept (this, o);
        s = s.getNext ();
      }    
    }
    fdef.getHead ().accept (this, o);
    return o;
  }

}
