package edu.toronto.cs.yasm.abstractor;


import edu.toronto.cs.yasm.refiner.*;
import edu.toronto.cs.yasm.pprogram.*;
import edu.toronto.cs.expr.*;
import edu.toronto.cs.yasm.YasmStatistics;
import edu.toronto.cs.yasm.wp.*;

import edu.toronto.cs.tp.SymbolTable;
import edu.toronto.cs.util.IdentityHashSet;

import edu.toronto.cs.cparser.CILTransformerUtil;

import java.util.*;

public class CBJPredicateRefiner extends PredicateRefiner
{

  Map cache;

  List<Expr> allPreds;

  boolean doInconsitent;
  boolean doSearch;


  boolean useRelatedPredicates = false;

  // For pretty-printing
  DisjunctionRewriter disjRewriter;

  public CBJPredicateRefiner (ExprFactory _fac, 
			      SymbolTable st)
  {
    super (_fac, st);
    doInconsitent = true;
    doSearch = true;
    disjRewriter = new DisjunctionRewriter ();

  }

  /**
   * doInconsitent -- if true compute inconsistent cubes
   * doSearch -- if true do abstraction search
   *
   * @param _fac an <code>ExprFactory</code> value
   * @param _doInconsitent a <code>boolean</code> value
   * @param _doSearch a <code>boolean</code> value
   */
  public CBJPredicateRefiner (ExprFactory _fac, SymbolTable st, 
			      boolean _doInconsitent,
			      boolean _doSearch)
  {
    super (_fac, st);

    doInconsitent = _doInconsitent;
    doSearch = _doSearch;
    disjRewriter = new DisjunctionRewriter ();

  }
  //   public CBJPredicateRefiner (ExprFactory _fac, List _newPredicates)
  //   {
  //     super (_fac, _newPredicates);
  //   }


  public PProgram doProgramRefine (PProgram p)
  {
    if (newPredicates.isEmpty () && !predicates.isEmpty ()) return p;

    // -- predicates is empty when this is the first run of this refiner
    if (predicates.isEmpty ())
      p.setInconsistent (fac.falseExpr ());

    cache = new IdentityHashMap ();

    allPreds = new ArrayList<Expr> ();
    allPreds.addAll (Pred.getNames (predicates));
    allPreds.addAll (Pred.getNames (newPredicates));


    if (!allPreds.isEmpty () && doInconsitent)
    {
      
      TPCBJPredicateAbstractor cbj = 
	new TPCBJPredicateAbstractor (allPreds,
				      tp.exprFactory ().falseExpr (),
				      tp);
      cbj.recSearch (1);
      Expr inconsistent = cbj.getTExpr ();
      System.err.println ("INCONSISTENT: " + inconsistent);
      p.setInconsistent (inconsistent);
    }

    return super.doProgramRefine (p);
  }

  private CBJRefinerInfo getRefinerInfo (PrllAsmtPStmt asmt)
  {
    CBJRefinerInfo info = (CBJRefinerInfo) asmt.getRefinerInfo ();
    if (info == null)
    {
      info = new CBJRefinerInfo ();
      asmt.setRefinerInfo (info);
    }
    return info;
  }

  public MemoryModel getRegularMemoryModel (PrllAsmtPStmt asmt)
  {
    return getRefinerInfo (asmt).memoryModel;
  }

  public CVCLMemoryModel getMemoryModel (PrllAsmtPStmt asmt)
  {
    return getRefinerInfo (asmt).cvclMemoryModel;
  }

  public WPComputer getWPComputer (PrllAsmtPStmt astmt)
  {
    return getRefinerInfo (astmt).wp;
  }

  public PrllAsmtPStmt doAsmtRefine (PrllAsmtPStmt asmt)
  {

    WPComputer wp = asmt.getWPComputer ();

    List asmts;
    int start;

    System.out.println (">>> doAsmtRefine: " + asmt);

    /**
     ** This is a bit confusing, but this is how it works:
     ** if doSearch is true, we go over all new and old predicates
     ** and compute the new abstraction of each statement. Therefore, 
     ** we start at the first predicate and an empty list

     ** If we do not do search, then we only need to compute new information
     ** for new predicates we have. Thus we start with previous computation
     ** which is stored in 'astm.getAsmts ()', and only process new predicates
     ** that start at the index 'predicates.size ()'
     **/
    if (doSearch)
    {
      start = 0;
      asmts = new ArrayList ();
    }
    else
    {
      asmts = asmt.getAsmts ();
      start = predicates.size ();
    }

    for (int i = start; i < allPreds.size (); i++)
    {
      Expr pred = (Expr) allPreds.get (i);

      stats.startWP ();
      Expr wpExpr = wp.computeWP ((Expr) allPreds.get (i));
      stats.stopWP ();

      System.err.println (">>> pred " + pred);
      System.err.println (">>> wp " + disjRewriter.rewrite (wpExpr));

      assert wpExpr != null : "null wp for: " + asmt;

      // XXX result[1] is positive choice condition, result[0] is negative
      Expr[] result = null;

      if (NondetOp.hasNondet (wpExpr))
	{
	  if (wpExpr.varSet (new HashSet ()).isEmpty ())
	    {
	      System.err.println ("Got predicate: " + 
				  allPreds.get (i));
	      System.out.println (" with wp: " + wpExpr);
	      System.out.println ("It should change non-deterministically");
	    }
	    
	  // -- this predicate is afffected by a 
	  // -- non-deterministic assignment
	    
	  // -- at the moment, just say we know nothing about it
	  result = new Expr[] { fac.falseExpr (), fac.falseExpr () };
	}

      if (result == null)
	{
	  
	  if (ChoiceOp.lgetT (wpExpr) == pred)
	    {
	      // -- positive condition is a predicate
	      //System.err.println ("QUICKTEST1");
	      result = 
		new Expr[] {fac.op (BoolOp.NOT).
			    unaryApply (pred), pred};
	      
	    }
	  else if (tp.quickQuery (ChoiceOp.lgetT (wpExpr)))
	    {
	      // -- positive condition is true
	      //System.err.println ("QUICKTEST2");
	      result = new Expr[] { fac.falseExpr (), fac.trueExpr () };
	    }
	  else if (tp.quickQuery (ChoiceOp.lgetF (wpExpr)))
	    {
	      // -- negative condition is true
	      //System.err.println ("QUICKTEST3");
	      result = new Expr[] 
		{ fac.trueExpr (), fac.falseExpr () };
	    }
	  else
	    result = (Expr[]) cache.get (wpExpr);
	}
      

      if (result == null && doSearch)
      {
        List<Expr> relPreds;
        if (useRelatedPredicates)
	  {
	    stats.startRelatedPredicates ();
	    relPreds = relatedPredicates (allPreds, wpExpr);
	    stats.stopRelatedPredicates ();
	  }
        else
	  {
	    relPreds = allPreds;
	  }

        if (useRelatedPredicates && relPreds.isEmpty ())
	  {
	    // no relevant predicates in predicate set
	    // so we have nothing to approximate wpExpr with
	    result = new Expr[] { fac.falseExpr (), fac.falseExpr () };
	  }
	else
	  {
	    
	    TPCBJPredicateAbstractor cbj = 
	      new TPCBJPredicateAbstractor (relPreds, wpExpr, tp);
	
	    if (stats != null)
	      stats.startPredicateAbstraction ();
	    cbj.recSearch (1);
	    if (stats != null)
	      stats.stopPredicateAbstraction ();
	    result = new Expr[] {cbj.getFExpr (), cbj.getTExpr ()};
	    cache.put (wpExpr, result);
	  }
      }
      else if (result == null && !doSearch)
        result = new Expr[] { fac.falseExpr (), fac.falseExpr () };
      else
        ; //System.err.println ("CACHE HIT");

      asmts.add 
        (new AsmtPStmt 
         ((Expr) allPreds.get (i), result [1], result [0]));
    }

    asmt.setAsmts (asmts);
    return asmt;
  }


  /** 
   * Returns a list of all the predicates in {@code preds} which could
   * possibly imply {@code target}.
   *
   * Currently constructs the set of related predicates by including a
   * predicate p from preds if p and target share a common variable and
   * transitively including predicates if they're related to related
   * predicates. More clever checks could further reduce the size of the
   * related predicate set.
   * 
   * @param preds 
   * @param target 
   * @return 
   */
  private static List<Expr> relatedPredicates (List<Expr> preds, Expr target)
  {
    List<Expr> tempList = new LinkedList<Expr> (); 
    tempList.add (target);
    List<Expr> result = relatedPredicates (preds, tempList);
    result.remove (target);
    return result;
  }

  private static List<Expr> relatedPredicates 
    (List<Expr> preds, List<Expr> relPreds)
  {
    if (relPreds.isEmpty ())
      return Collections.EMPTY_LIST;

    List<Expr> newRelPreds = new LinkedList<Expr> ();

    for (Expr pred : preds)
      for (Expr target : relPreds)
        if (commonVars (pred, target) && !relPreds.contains (pred))
          newRelPreds.add (pred);

    if (!newRelPreds.isEmpty ())
    {
      relPreds.addAll (newRelPreds);
      return relatedPredicates (preds, relPreds);
    }

    return relPreds;
  }

  /** 
   * Returns true iff {@code p1} and {@code p2} contain a common variable. 
   * 
   * @param p1 
   * @param p2 
   * @return 
   */
  private static boolean commonVars (Expr p1, Expr p2)
  {
    // check for non-empty intersection:
    // where A intersect B != emptyset
    // iff. A != A - B

    // Set isn't closed under removeAll :(
    Set diff = vars (p1);
    diff.removeAll (vars (p2));

    if (!vars (p1).equals (diff))
      return true;

    return false;
  }

  /** 
   * Returns a Set containing all the variables in {@code e}. 
   * 
   * @param e 
   * @return 
   */
  private static Set<Expr> vars (Expr e)
  {
    return vars (e, new IdentityHashSet ());
  }

  private static Set<Expr> vars (Expr e, Set s)
  {
    if (e.arity () == 0 && VariableOp.isVar (e))
      s.add (e);
    else
      for (Expr child : e.args ())
        s.addAll (vars (child, s));

    return s;
  }

  public class CBJRefinerInfo
  {
    MemoryModel memoryModel;
    CVCLMemoryModel cvclMemoryModel;
    WPComputer wp;

    // -- more stuff
  }

  public static void main (String[] args) throws Exception
  {
    ExprFactory fac = new ExprFactoryImpl ();
    // test vars
    Expr e = CILTransformerUtil.getExpr (fac, "&x == &y && *z > (a + b)");
    System.out.println ("e: " + e);
    System.out.println ("vars in e: " + vars (e));

    // test isRelatedPredicates
    Expr p1 = CILTransformerUtil.getExpr (fac, "&a > 0");
    Expr p2 = CILTransformerUtil.getExpr (fac, "x > 0");
    Expr p3 = CILTransformerUtil.getExpr (fac, "z > 0");
    Expr p4 = CILTransformerUtil.getExpr (fac, "c > 0");
    Expr p5 = CILTransformerUtil.getExpr (fac, "&a == &e");
    // pred which should be included by transitivity, but is not directly
    // related
    Expr p6 = CILTransformerUtil.getExpr (fac, "e > 0");
    System.out.println (p1 + " related to e " + commonVars (p1, e));
    System.out.println (p2 + " related to e " + commonVars (p2, e));
    System.out.println (p3 + " related to e " + commonVars (p3, e));
    System.out.println (p4 + " related to e " + commonVars (p4, e));
    System.out.println (p5 + " related to e " + commonVars (p5, e));
    System.out.println (p6 + " related to e " + commonVars (p6, e));

    // test relatedPredicates

    List<Expr> preds = new LinkedList<Expr> ();
    preds.add (p1);
    preds.add (p2);
    preds.add (p3);
    preds.add (p4);
    preds.add (p5);
    preds.add (p6);
    System.out.println ("related to e");
    System.out.println (relatedPredicates (preds, e));

  }
}
