package edu.toronto.cs.yasm.abstractor;

import edu.toronto.cs.cparser.block.*;
import edu.toronto.cs.cparser.*;
import edu.toronto.cs.expr.*;
import edu.toronto.cs.yasm.*;
import edu.toronto.cs.yasm.pprogram.*;
import edu.toronto.cs.yasm.YasmStatistics;

import edu.toronto.cs.tp.cvcl.CVCLUtil;
import edu.toronto.cs.tp.cvcl.ValidityChecker;
import edu.toronto.cs.tp.*;

import edu.toronto.cs.boolpg.abstraction.StmtAbstraction;

import edu.toronto.cs.yasm.wp.*;

import antlr.*;

import static edu.toronto.cs.expr.CVarDeclOp.*;

import java.util.*;
import java.io.*;

public class PredicateRefiner implements Refiner
{
  YasmStatistics stats;

  List predicates; // e.t.c.yasm.pprogram.Pred's
  List newPredicates; // e.t.c.yasm.pprogram.Pred's

  // -- our theorem prover
  ITheoremProver tp;

  // -- expression factory to construct expressions, also used by tp
  ExprFactory fac;

  // DeclsParser declsParser;
  // SymbolicExecutor symbolicExecutor;

  // -- marks whether a PProgram component has been refined already
  Map isRefined;

  Expr unknownCond;

  public PredicateRefiner (ExprFactory fac)
  {
    throw new UnsupportedOperationException ();
  }

  public PredicateRefiner (ExprFactory fac, List newPredicates)
  {
    throw new UnsupportedOperationException ();
  }

  public PredicateRefiner (ExprFactory fac, SymbolTable st)
  {
    this (fac, new ArrayList (), st);
  }

  public PredicateRefiner (ExprFactory fac, List newPredicates, SymbolTable st)
  {
    this.fac = fac;
    this.newPredicates = newPredicates;

    predicates = new ArrayList ();
    addToNewPredicates (newPredicates);

    // -- create an unknown variable not used anywhere else
    // XXX magic var -- declared as int in symbol table
    Expr unknownVar = fac.var ("__UNKNOWN__");

    // -- make unknown if-condition -- a condition that can never
    // -- be shown to be neither true nor false
    unknownCond = fac.op (ComparisonOp.EQ).binApply
      (unknownVar, fac.intExpr (0));

    tp = new CTheoremProver (fac, new CLCVCLTheoremProver (fac, true), st);

    isRefined = new IdentityHashMap ();
  }

  public List getPredicates ()
  {
    return Pred.getNames (predicates);
  }

  public List getNewPredicates ()
  {
    return Pred.getNames (newPredicates);
  }

  public List getPreds ()
  {
    return predicates;
  }

  public List getNewPreds ()
  {
    return newPredicates;
  }


  public ExprFactory exprFactory ()
  {
    return tp.exprFactory ();
  }

  /**
   * Returns a copy of {@code ITheoremProver} used by this refiner
   *
   * @return an instance of <code>ITheoremProver</code> 
   */
  public ITheoremProver getTP ()
  {
    return tp;
  }

  public ValidityChecker getVC ()
  {
    throw new UnsupportedOperationException ();
    // return tp.getValidityChecker ();
  }

  public CVCLExprConverter getCVCLConverter ()
  {
    throw new UnsupportedOperationException ();
    // return tp.getExprConverter ();
  }

  public void addToNewPredicates (List l)
  {
    isRefined = new IdentityHashMap (); // all nodes to be refined again

    for (Iterator it = l.iterator (); it.hasNext ();)
      addToNewPredicates ((Pred) it.next ());
  }

  public void addToNewPredicates (Pred p)
  {
    isRefined = new IdentityHashMap (); // all nodes to be refined again

    // -- only add predicates which are not already in newPredicates
    if (!newPredicates.contains (p))
      newPredicates.add (p);
    else
      System.err.println ("Refiner: addToNewPredicates: Not adding " +
          "existing predicate: " + p);
  }

  /** 
   * @deprecated Use addToNewPredicates
   */
  public void addPredicates (List l)
  {
    addToNewPredicates (l);
  }

  /** 
   * @deprecated Use addToNewPredicates
   */
  public void addPredicate (Pred p)
  {
    addToNewPredicates (p);
  }

  public PProgram doProgramRefine (PProgram p)
  {
    assert p.isDeclsRefined () == true
      : "Decls not refined; cannot refine PProgram";

    // -- refine each function
    for (Iterator it = p.getFunctionDefs ().values ().iterator ();
        it.hasNext ();)
      doFunctionRefine ((PFunctionDef) it.next ());

    // -- add new predicates to PProgram
    for (Iterator it = newPredicates.iterator (); it.hasNext(); )
      p.getPredTable ().newPredicate (((Pred) it.next ()).getName ());

    // -- add new predicates to Refiner (this)
    predicates.addAll (newPredicates);

    // -- reset new predicates
    newPredicates = new ArrayList ();

    return p;
  }


  public void doDeclsRefine (PProgram p)
  {
    assert false;
  }
  
  public boolean hasNewPredicates ()
  {
    return !newPredicates.isEmpty ();
  }

  public PFunctionDef doFunctionRefine (PFunctionDef f)
  {
    // -- refine the body
    f.setHead (doInsideFunctionRefine (f.getHead ()));

    return f;
  }

  public PStmt doInsideFunctionRefine (PStmt s)
  {
    if (s == null)
      return null;

    PStmt refinedStmt;    

    if (!isRefined.containsKey (s))
    {
      // have not refined this node on this refinement pass; remember that  
      // we've visited this node
      isRefined.put (s, s);

      if (s instanceof IfPStmt)
        refinedStmt = doIfRefine ((IfPStmt) s);
      else if (s instanceof PrllAsmtPStmt)
        refinedStmt = doAsmtRefine ((PrllAsmtPStmt) s);
      else if (s instanceof GotoPStmt)
        refinedStmt = doGotoRefine ((GotoPStmt) s);
      else if (s instanceof ReturnSelectorPStmt)
        refinedStmt = doReturnSelectorRefine ((ReturnSelectorPStmt) s);
      else if (s instanceof NDGotoPStmt)
        refinedStmt = doNDGotoRefine ((NDGotoPStmt) s);
      else if (s instanceof SkipPStmt)
        refinedStmt = doSkipRefine ((SkipPStmt) s);
      else if (s instanceof FunctionCallPStmt)
        refinedStmt = doFunctionCallRefine ((FunctionCallPStmt) s);
      else if (s instanceof ConditionalBranchPStmt)
        refinedStmt = doConditionalBranchRefine ((ConditionalBranchPStmt) s);
      else if (s instanceof SwitchBranchPStmt)
        refinedStmt = doSwitchBranchRefine ((SwitchBranchPStmt) s);
      else
	throw new RuntimeException ("Can't refine: " + s.getClass ());   

      refinedStmt.setNext (doInsideFunctionRefine (s.getNext ()));

      return refinedStmt;
    }
    else
    {
      System.out.println ("Saw " + s + " twice");
      // node was refined on this pass already
      s.setNext (doInsideFunctionRefine (s.getNext ()));
      return s;
    }

  }

  public IfPStmt doIfRefine (IfPStmt s)
  {
    // System.err.println (">>> doIfRefine " + s.getLabel ());
    s.setCond (doCondRefine (s.getCond ()));
    s.setThenStmt (doInsideFunctionRefine (s.getThenStmt ()));
    s.setElseStmt (doInsideFunctionRefine (s.getElseStmt ()));
    return s;
  }

  public ConditionalBranchPStmt doConditionalBranchRefine
    (ConditionalBranchPStmt s)
  {
    s.setCond (doCondRefine (s.getCond ()));
    return s;
  }

  public SwitchBranchPStmt doSwitchBranchRefine (SwitchBranchPStmt s)
  {
    // XXX Currently does nothing--assumes preRefine sets the abstract
    // condition for a switch
    return s;
  }

  // -- returns true if every operator/operand in expr is known
  // XXX this should be checked elsewhere, e.g., in CTheoremProver,
  // CToAddrModel, CLCVCLTheoremProver, or CVCLExprPrinter, not here, since
  // those classes define Expr->CVCL conversion
  public boolean knownCond (Expr expr)
  {
    if (expr == null)
      return false;

    if (isCVarDecl (expr))
      return true;

    if (expr.op () instanceof VariableOp)
      return true;

    if (expr.op () instanceof RationalOp)
      return true;

    if (expr.op () instanceof JavaObjectOp)
      return true;

    if (expr.op () instanceof ArrayOp)
	 return true;

    if (expr.op () instanceof BoolOp ||  
        (expr.op () instanceof NumericOp &&
         expr.op () != NumericOp.MOD) ||
        expr.op () instanceof ComparisonOp ||
        expr.op () instanceof CILIndirectionOp)
    {
      for (int i = 0; i < expr.arity (); i++)
        if (!knownCond (expr.arg (i)))
          return false;
      return true;
    }


    return false;  
  }

  public PCond doCondRefine (PCond e)
  {
    // System.err.println ("doCondRefine: " + e);

    // -- check that condition is as refined as can be already
    // -- XXX currently, each condition can be refined at most once
    // -- XXX at which point it's value changed from IBOT to something 
    // -- XXX else. 
    if (e.getCond ().op () != BiLatticeOp.IBOT)
      return e;

    // -- NOTE: YasmApp.preRefine now sets cond's abstracted condition to the
    // if-condition itself

    try
    {
      Expr sourceExpr = (Expr) e.getOrigCond ();

      // -- unable to convert to cvcl, e.g. contains an unknown operator
      if (!knownCond (e.getOrigCond ()))
      {
        System.out.println (">>> unknown condition: " + e.getOrigCond ());
        e.setOrigCond (unknownCond);
      }


      sourceExpr = e.getOrigCond ();  

      // -- check if the condition is a constant
      Boolean result = tp.quickCheckTruth (sourceExpr);

      // -- result != null iff original condition is a constant
      if (result != null)
      {
        e.setCond 
          (result == Boolean.TRUE ? fac.trueExpr () : fac.falseExpr ());
        return e;
      }

      // -- check if condition is negated and strip the negation
      boolean negated = sourceExpr.op () == BoolOp.NOT;
      Expr condExpr = negated ? sourceExpr.arg (0) : sourceExpr;

      // -- see if the condition is equivalent to one of our 
      // -- new predicates
      for (Iterator it = newPredicates.iterator (); it.hasNext ();)
      {
        Expr pred = ((Pred) it.next ()).getName ();
        if (condExpr == pred /* || 
                                CVCLUtil.quickQuery (vc, 
                                vc.iffExpr (cvcl.toCVCL (condExpr),
                                cvcl.toCVCL (pred))) */)
        {
          // -- got a new predicate equivalent to the condition
          e.setCond (negated ? 
              fac.op (BoolOp.NOT).unaryApply (pred) : pred);
          return e;
        }

      }
    }
    catch (Exception ex)
    {
      ex.printStackTrace ();
      throw new RuntimeException (ex);
    }
    return e;

  }

  public CVCLMemoryModel getMemoryModel (PrllAsmtPStmt asmt)
  {
    return (CVCLMemoryModel) asmt.getRefinerInfo ();
  }

  public WPComputer getWPComputer (PrllAsmtPStmt asmt)
  {
    return getPredRefinerInfo (asmt).wp;
  }

  private PredicateRefinerInfo getPredRefinerInfo (PrllAsmtPStmt asmt)
  {
    PredicateRefinerInfo info = (PredicateRefinerInfo) asmt.getRefinerInfo ();
    if (info == null)
    {
      info = new PredicateRefinerInfo ();
      asmt.setRefinerInfo (info);
    }
    return info;
  }

  public MemoryModel computeRegularMemoryModel (PrllAsmtPStmt s)
  {
    assert false : "Use getWPComputer!";

    return null;
  }

  public CVCLMemoryModel computeMemoryModel (PrllAsmtPStmt s)
  {
    assert false : "Use getWPComputer!";
    return null;
  }

  public PrllAsmtPStmt doAsmtRefine (PrllAsmtPStmt s)
  {
    assert false : "Shouldn't be using base refiner!";
    return null;
  }

  public GotoPStmt doGotoRefine (GotoPStmt s)
  {
    return s;
  }

  public ReturnSelectorPStmt doReturnSelectorRefine (ReturnSelectorPStmt s)
  {
    return s;
  }

  public NDGotoPStmt doNDGotoRefine (NDGotoPStmt s)
  {
    return s;
  }

  public SkipPStmt doSkipRefine (SkipPStmt skip)
  {
    return skip;
  }

  public FunctionCallPStmt doFunctionCallRefine (FunctionCallPStmt fcall)
  {
    FunctionCallPrologue prologue = (FunctionCallPrologue) fcall.getLogues ();
    doAsmtRefine (prologue);

    FunctionCallEpilogue epilogue =
      (FunctionCallEpilogue) fcall.getLogues ().getNext ();
    doAsmtRefine (epilogue);  

    return fcall;
  }

  /** 
   * Parses stream read from Reader; each predicate parsed is added to
   * the list of predicates.
   * 
   * @param in Source of predicates.
   */
  public void readPredicates (Reader in) 
    throws RecognitionException, TokenStreamException 
    {
      CILLexer lexer = new CILLexer (in);
      lexer.setTokenObjectClass (CToken.class.getName ());

      PredicateParser parser = new PredicateParser (lexer);
      parser.setASTNodeClass (Block.class.getName ());
      Block.setTokenVocabulary (CILTokenTypes.class.getName ());

      parser.parse();

      Block cPred = (Block) parser.getAST ();
      while (cPred != null)
      {
        /*
           System.err.println ("attempting to parse predicate:");
           TNode.setTokenVocabulary (CILTokenTypes.class.getName ());
           TNode.printTree (cPred);
           */

        Expr pred = SymbolicExecutorUtil.newSymbolicExecutor (fac).pureExpr
          (cPred, new MemoryModel ());
        System.err.println ("Adding predicate: " + pred);
        // XXX just assume it's from a hint
        addToNewPredicates (new Pred (pred, PredicateSource.HINT));

        cPred = (Block) cPred.getNextSibling ();
      }
    }

  protected List computeDependency (List l, Expr e, Map m)
  {
    List temp = new ArrayList ();

    for (Iterator it = l.iterator (); it.hasNext ();)
    {
      Expr expr = (Expr) it.next ();
      List dependingVars = (List)m.get (expr);

      assert dependingVars!=null;

      if (contains (e, dependingVars))
        temp.add (expr);
    }
    return temp;
  }

  protected boolean contains (Expr e, List l)
  {
    if (e.arity () == 0)
    {
      for (Iterator it = l.iterator (); it.hasNext ();)
        if (e.equals ((Expr)it.next ()))
          return true;
    }
    else 
      for (int i = 0; i < e.arity (); i++)
        return contains (e.arg (i), l);
    return false;
  }

  protected List findVars (Expr e)
  {
    List l = new LinkedList ();
    if (e.arity () == 0)
    {
      if (e.op () instanceof VariableOp)
        l.add (e);
    }
    else
      for (int i = 0; i < e.arity (); i++)
      {
        l.addAll (findVars (e.arg (i)));
      }
    return l;
  }

  class PredicateRefinerInfo 
  {
    CVCLMemoryModel mmModel;
    WPComputer wp;
  }

  /**
   * Sets a stats object to collect statistics
   *
   * @param stats an <code>YasmStatistics</code> value
   */
  public void setStats (YasmStatistics stats)
  {
    this.stats = stats;
    if (tp instanceof CTheoremProver)
      ((CTheoremProver) tp).setStats (stats);
  }
}

