package edu.toronto.cs.yasm.abstractor;

import edu.toronto.cs.expr.*;


import edu.toronto.cs.yasm.*;
import edu.toronto.cs.yasm.pprogram.*;


import edu.toronto.cs.tp.ITheoremProver;
import edu.toronto.cs.tp.cvcl.ValidityChecker;


import edu.toronto.cs.yasm.wp.*;

import antlr.*;
import java.util.*;
import java.io.*;

public interface Refiner
{
  /**
   * Refines a predicate program with new predicates
   *
   * @param p an input predicate program
   * @return a refined version of <code>p</code>
   */
  PProgram doProgramRefine (PProgram p);

  /**
   * Collects declarations from p and declares them in the theorem prover
   *
   * @param p a <code>PProgram</code> value
   */
  void doDeclsRefine (PProgram p);

  /**
   * Sets a stats object to collect statistics
   *
   * @param stats an <code>YasmStatistics</code> value
   */
  void setStats (YasmStatistics stats);
  
  /**
   * Reads a list of predicates from an input file
   *
   * @param in a <code>Reader</code> value
   */
  void readPredicates (Reader in)
    throws RecognitionException, TokenStreamException;
  


  /**
   * return predicates used for refinement
   *
   * @return a <code>List</code> value
   */
  List getPredicates ();

  /**
   * return predicates used for refinement, in @code{Pred} form
   *
   * @return a <code>List</code> value
   */
  List<Pred> getPreds ();
  

  /**
   * return new predicates used for refinment
   *
   * @return a <code>List</code> value
   */
  List getNewPredicates ();

  /**
   * return new predicates used for refinment, in @code{Pred} form
   *
   * @return a <code>List</code> value
   */
  List<Pred> getNewPreds ();


  /**
   * return the validity checker used for refinment
   *
   * @return a <code>ValidityChecker</code> value
   */
  ValidityChecker getVC ();

  ITheoremProver getTP ();
  
  ExprFactory exprFactory ();


  /**
   * return CVCLite converter
   *
   * @return a <code>CVCLExprConverter</code> value
   */
  CVCLExprConverter getCVCLConverter ();
  

//   /**
//    * add new predicates to predicate list
//    *
//    * @param l a <code>List</code> value
//    */
//   void addToNewPredicates (List l);
  


  /**
   * add new predicates to predicate list
   *
   * @param p an <code>Expr</code> value
   */
   void addToNewPredicates (Pred p);
  
  /** 
   * @deprecated Use addToNewPredicates
   */
  void addPredicates (List<Pred> l);


   /** 
    * @deprecated Use addToNewPredicates
    */
   void addPredicate (Pred p);


//   /**
//    * check if there are new predicates for refinement
//    *
//    * @return a <code>boolean</code> value
//    */
//   boolean hasNewPredicates ();

  /**
   * return CVCL Memory Model
   *
   * @param asmt a <code>PrllAsmtPStmt</code> value
   * @return a <code>CVCLMemoryModel</code> value
   */
  CVCLMemoryModel getMemoryModel (PrllAsmtPStmt asmt);


  /**
   * retrun WP computer used for refinement
   *
   * @param asmt a <code>PrllAsmtPStmt</code> value
   * @return a <code>WPComputer</code> value
   */
  WPComputer getWPComputer (PrllAsmtPStmt asmt);

//   /**
//    * create Memory Model
//    *
//    * @param s a <code>PrllAsmtPStmt</code> value
//    * @return a <code>MemoryModel</code> value
//    */
//   MemoryModel computeRegularMemoryModel (PrllAsmtPStmt s);


//   /**
//    * create CVCL memory model
//    *
//    * @param s a <code>PrllAsmtPStmt</code> value
//    * @return a <code>CVCLMemoryModel</code> value
//    */
//  CVCLMemoryModel computeMemoryModel (PrllAsmtPStmt s);

}
