package edu.toronto.cs.yasm.cfg;

import java.util.List;

import edu.toronto.cs.yasm.cfg.CFGraph.BasicBlock;
import edu.toronto.cs.expr.*;



public class SimpleBlockSimplifier implements CFGSimplifier
{
  public SimpleBlockSimplifier ()
  {
    super ();
  }
  
  
  /**
   * A static convinience method to run this simplifier
   * 
   * @param cfg CFG to simlify
   * @return simplified cfg
   */
  public static CFGraph run (CFGraph cfg)
  {
    return new SimpleBlockSimplifier ().simplify (cfg);
  }
  

  public CFGraph simplify (CFGraph cfg)
  {
    for (BasicBlock bb : cfg.allBlocksFwd ())
      {
	Expr splitStmt = findSplitPt (bb.getStmts ());
	if (splitStmt != null)
	  bb.splitAtStmt (splitStmt);
      }
    
    return cfg;
  }
  

  /**
   * Finds a split point in the list of statments.  A split point is:
   * (a) a branching instruction preceeded by anything other than a
   * label; (b) a function call preceeeded by anything other than a
   * label; (c) any expresion following a function call.
   * 
   * @param stmts a list of statements
   * @return an element of the list to split at, or null if no split
   * point is found.
   */
  private Expr findSplitPt (List<Expr> stmts)
  {
    // -- a split point is 
    // -- 1) a conditional preceeded by anything other than a label
    // -- 2) a function call preceeded by anything other than a label
    // -- 3) anything following a function call

    boolean prevIsLabel = true;
    boolean prevIsFunc = false;
    
    for (Expr e : stmts)
      {
	
	// -- check for splitting points
	if (isBranching (e) && !prevIsLabel) return e;
	if (isFunctionCall (e) && !prevIsLabel)
	  return e;
	if (prevIsFunc) return e;

	// -- update previous values
	if (prevIsLabel && !CStmtOp.isLabelExpr (e)) 
	  prevIsLabel = false;
	prevIsFunc = isFunctionCall (e);
	
      }

    return null;
  }


  /**
   * Returns true if an expression is either a conditional or a switch 
   * 
   * @param e an expression
   * @return true if e is conditional or a switch
   */
  private boolean isBranching (Expr e)
  {
    return CStmtOp.isCondExpr (e) ||
      CSwitchExprOp.isSwitchExpr (e);
  }
  
  /**
   * Returns true if an expression contains a function call.
   * 
   * @param an expression
   * @return true if either e is a function call or an assignment
   * whose RHS is a function call.
  */
  private boolean isFunctionCall (Expr e)
  {
    return CCallExprOp.isCCallExpr (e)  ||
      CStmtOp.isModifyExpr (e) && isFunctionCall (e.arg (1));
  }
  


  

}
