package edu.toronto.cs.yasm.junit;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import edu.toronto.cs.algebra.AlgebraValue;
import edu.toronto.cs.ctl.antlr.CTLNodeParser;
import edu.toronto.cs.ctl.antlr.CTLNodeParser.CTLNodeParserException;
import edu.toronto.cs.yasm.YasrApp;
import junit.framework.TestCase;

/**
 * Describe class YasrTestCase here.
 *
 *
 * Created: Wed Dec 7 18:25:28 2005
 *
 * @author <a href="mailto:owei@age.cs">Ou Wei</a>
 * @version 1.0
 */
public class YasrTestCase extends GenericTestCase 
{

  String expectedStr;

  public YasrTestCase () {
    super();
    // 		root = System.getProperty("ROOT");
    // 		if (root == null) root = "";
  }

	
  public YasrTestCase (String name) {
    super(name);
    // 		root = System.getProperty("ROOT");
    // 		if (root == null) root = "";
  }
	
	
  /* Reading configuration from a property file to setup
   * the initial configuration of YasrApp
   */
	
  protected void loadProperties (YasrApp yasrApp, Properties props) 
    throws CTLNodeParser.CTLNodeParserException 
  {
    String filename = root + "/" + props.getProperty("file.name");

    // -- YasrApp is not BpFile. Instead, it uses cFile to save the filename
    // 		yasrApp.setBpFile(new File(filename));
    yasrApp.setCFile(new File(filename));

    String property = props.getProperty("property");
    yasrApp.setProp(CTLNodeParser.parse(property));
    String once = props.getProperty("once");
    if (once != null) {
      yasrApp.setDoProof(!(new Boolean (once).booleanValue()));
    }
    String hyper = props.getProperty("hyper");
    if (hyper != null) {
      yasrApp.setHyper (new Boolean (hyper).booleanValue());
    }

    // -- YasrApp is not BpFile. Instead, it uses cFile to save the filename
    // 		String cFile = props.getProperty("c.file");
    // 		if (cFile != null) {
    // 			yasrApp.setCFile(new Boolean (cFile).booleanValue());
    // 		}

    String ndInit = props.getProperty("nd.init");
    if (ndInit != null) {
      yasrApp.setUnknownInit(!(new Boolean (ndInit).booleanValue()));
    }
    String refinerType = props.getProperty("refiner.type");
    if (refinerType != null) {
      yasrApp.setRefinerType(refinerType);
    }
    String pgType = props.getProperty("p.generator.type");
    if (pgType != null) {
      yasrApp.setPGeneratorType(pgType);
    }
    String initPred = props.getProperty("init.pred");
    if (initPred != null) {
      yasrApp.setInitPredFile (new File (root + "/" + initPred));
    }
    else {
      yasrApp.setInitPredFile(null);
    }

    String selectorType = props.getProperty ("selector.type");
    if (selectorType != null)
      {
	yasrApp.setSelectorType (selectorType);
      }
    else
      {
	yasrApp.setSelectorType ("dd");
      }

    expectedStr = props.getProperty ("expected.value");
		
  }

  public void doTest() {
    System.out.println ("////////////////////////////////////////////");
    System.out.println ("////////////////////////////////////////////");
    System.out.println ("////////////////////////////////////////////");
    System.out.println ("///                                      ///");
    System.out.println ("///                                      ///");
    System.out.println ("///    Test " + getName() + " starts     ///");
    System.out.println ("///                                      ///");
    System.out.println ("///                                      ///");
    System.out.println ("////////////////////////////////////////////");
    System.out.println ("////////////////////////////////////////////");
    System.out.println ("////////////////////////////////////////////");
    System.err.println ("////////////////////////////////////////////");
    System.err.println ("////////////////////////////////////////////");
    System.err.println ("////////////////////////////////////////////");
    System.err.println ("///                                      ///");
    System.err.println ("///                                      ///");
    System.err.println ("///    Test " + getName() + " starts     ///");
    System.err.println ("///                                      ///");
    System.err.println ("///                                      ///");
    System.err.println ("////////////////////////////////////////////");
    System.err.println ("////////////////////////////////////////////");
    System.err.println ("////////////////////////////////////////////");
    AlgebraValue result = null;
    try 
      {
	YasrApp yasrApp = new YasrApp();
	loadProperties(yasrApp, properties);
	result = yasrApp.run();
      }
    catch (RuntimeException ex)
      {
	throw ex;
      }
    
    catch (CTLNodeParserException ex) 
      {
	YasrApp.out.println("Could not parse CTL property: " + ex);
	fail ("Could not parse CTL property: " + ex);
      } 
    catch (Throwable ex) 
      {
	YasrApp.out.println("Unhandled Exception: " + ex);
	ex.printStackTrace();
	fail ("Unhandled Exception: " + ex);
      }
    YasrApp.out.flush();
    YasrApp.err.flush();
    System.out.println ("Generic: result is: " + result + ".");
    assertNotNull(result);
    assertEquals (expectedStr, result.toString ());
  }
	
}
