package edu.toronto.cs.yasm.pcfa;

import edu.toronto.cs.mvset.*;
import edu.toronto.cs.util.*;

/**
 * Describe class Edge here.
 *
 *
 * Created: Wed Jun  8 13:03:07 2005
 *
 * @author <a href="mailto:arie@eon.cs">Arie Gurfinkel</a>
 * @version 1.0
 */
public abstract class Edge 
{
  Node.NodeId src;
  Node.NodeId dst;

  /**
   * Creates a new <code>Edge</code> instance.
   *
   */
  public Edge (Node.NodeId _src, Node.NodeId _dst) 
  {
    src = _src;
    dst = _dst;
  }

  protected Edge ()
  {
  }
  

  public Node.NodeId getSrc ()
  {
    return src;
  }
  public Node.NodeId getDst ()
  {
    return dst;
  }
  
  public Node.NodeId[] getAllDst ()
  {
    return new Node.NodeId [] { dst };
  }

  public boolean hasDst (Node.NodeId id)
  {
    return dst.equals (id);
  }
  
  
  /**
   * Does a bwdImage of a NodeSet over this edge.
   * 
   *
   * @param oldNodes a <code>NodeSet</code> a node set from which 
   * bwd image is done
   * @param newNodes a <code>NodeSet</code> a node set storing the result
   * @return a <code>NodeSet</code> value
   */
  public abstract boolean bwdImage (NodeSet oldNodes, 
				    NodeSet newNodes, 
				    boolean isDisjoin);

  public abstract boolean fwdImage (NodeSet oldNodes, NodeSet newNodes);
  //blic abstract boolean dualBwdImage (NodeSet oldNodes, NodeSet newNodes);

  public boolean equals (Edge e)
  {
    return e.getSrc () == getSrc () && e.getDst () == getDst ();
  }
  
  public int hashCode ()
  {
    long hash = getSrc ().hashCode () * Primes.getPrime (1)
      + getDst ().hashCode ();
    return (int)hash;
  }

  public String toString ()
  {
    return getSrc () + " -> " + getDst ();
  }
  
  

}
