package edu.toronto.cs.yasm.pgenerator;

import java.util.*;

import edu.toronto.cs.tp.ITheoremProver;
import edu.toronto.cs.expr.Expr;
import edu.toronto.cs.expr.ExprFactory;
import edu.toronto.cs.expr.BoolOp;
import edu.toronto.cs.yasm.pprogram.PProgram;
import edu.toronto.cs.yasm.pprogram.PFunctionDef;
import edu.toronto.cs.yasm.pprogram.PrllAsmtPStmt;
import edu.toronto.cs.yasm.pprogram.AsmtPStmt;
import edu.toronto.cs.yasm.wp.*;
import edu.toronto.cs.yasm.abstractor.Refiner;

import static edu.toronto.cs.expr.BoolOp.*;

public class InconsistentDestinationGenerator extends BasePredicateGenerator
{
  
  PFunctionDef pfd;

  public InconsistentDestinationGenerator (PProgram _pProgram,
					   Refiner _refiner,
					   List<Expr> _cStatePred)
    
  {
    super (_pProgram, null,_refiner, null, null, _cStatePred);    
    pfd = null;
  }


  public InconsistentDestinationGenerator (PProgram _pProgram,
					   Refiner _refiner,
					   List<Expr> _cStatePred,
                                           PFunctionDef _pfd)
    
  {
    super (_pProgram, null,_refiner, null, null, _cStatePred);    
    pfd = _pfd;
  }


  public boolean find ()
  {
    int scope = tp.stackLevel ();
    try
      {
	// -- push current scope and assume constraints of 
	// -- the parent state
	tp.push ();
	tp.assertFormula (cStatePred);
	// -- make sure we are going to a consistent state
	return isInconsistent ();
      }
    finally
      {
	tp.popto (scope);
      }
  }

  boolean isInconsistent ()
  {
    int scope = tp.stackLevel ();
    try
      {
	tp.push ();
	// -- if everything is consistent nothing to do
	if (!tp.query (tp.exprFactory ().falseExpr ())) 
	  return false;

	Expr newInconsistent = and (tp.getAssumptionsUsed ());

	System.err.println (this.getClass () + 
			    ": New inconsistent constraint: " + 
			    newInconsistent);
	
	// -- 
	PProgram prog = getPProgram ();
	ExprFactory fac = prog.getInconsistent ().getFactory ();
	prog.
	  setInconsistent (fac.op (BoolOp.OR).
			  binApply (prog.getInconsistent (),
				    newInconsistent));
        if (pfd != null)
          pfd.setInconsistent (or (pfd.getInconsistent (), newInconsistent));

	return true;
      }
    finally
      {
	tp.popto (scope);
      }
  }
}
