package edu.toronto.cs.yasm.pprogram;

import edu.toronto.cs.expr.*;


import edu.toronto.cs.yasm.abstractor.*;

import java.util.*;
import java.io.*;


/**
 * Given an expression representing a predicate, remove all {@link
 * edu.toronto.cs.yasm.pprogram.AsmtPStmt}s that assign to our
 * expression.
 *
 *
 * @author <a href="mailto:tomhart@cs.toronto.edu">Tom Hart</a>
 * @version 1.0
 */
public class AsmtRemoverVisitor extends BasePProgramVisitor 
{
  Expr killMe;

  public AsmtRemoverVisitor (Expr _killMe) 
  {
    super ();
    killMe = _killMe;
  }

  public Object visitPProgram (PProgram p, Object o)
  {
    for (PFunctionDef pf : p.getFunctionDefs ().values ()) {
      pf.accept (this, o);
    }
    return o;
  }
  

  public Object visitFunctionDef (PFunctionDef fdef, Object o)
  {
    if ( fdef.getHead () != null) {	
      PStmt s = fdef.getEntryPoint (); 
      while (s != null) {
        s.accept (this, o);
        s = s.getNext ();
      }
    }    

    fdef.getHead ().accept (this, o);

    return o;
  }

  public Object visitIf (IfPStmt s, Object o)
  {
    if (s.getThenGoto () != null) {
	PStmt ts = s.getThenStmt ();

	while (ts != null) {
          ts.accept (this, o);
          ts = ts.getNext();
        }
    }

    if (s.getElseGoto () != null) {
	PStmt ts = s.getElseStmt ();

	while (ts != null) {
          ts.accept (this, o);
          ts = ts.getNext();
        }
    }
    
    return o;
    
  }


  public Object visitPrllAsmt (PrllAsmtPStmt s, Object o)
  {
    s.removeAsmtsToVar (killMe);
    return o;     
  }
  
  public Object visitFunctionCallPrologue (FunctionCallPrologue s, Object o)
  {
    return visitPrllAsmt (s, o);
  }

  public Object visitFunctionCallEpilogue (FunctionCallEpilogue s, Object o)
  {
    return visitPrllAsmt (s, o);
  }





}
