package edu.toronto.cs.yasm.pprogram;

import edu.toronto.cs.cparser.block.Block;
import java.io.*;
import edu.toronto.cs.expr.*;
import edu.toronto.cs.yasm.abstractor.*;
import antlr.RecognitionException;
import java.util.*;

import static edu.toronto.cs.expr.CILFunctionCallOp.*;
import static edu.toronto.cs.expr.CCallExprOp.*;

public class FunctionCallPStmt extends PStmt
{
  // -- identifies this call statement
  int callIndex;

  // -- prologue and epilogue (linked by next)
  PStmt logues;

  // -- definition of function this statement is calling
  PFunctionDef functionDef;

  // -- variable where return value might be stored
  Expr returnVar;

  public FunctionCallPStmt (PFunctionDef parent, Block sourceBlock)
  {
    super (parent, sourceBlock);
    callIndex = sourceBlock.getCallIndex ();
  }

  public FunctionCallPStmt (PFunctionDef parent, Expr sourceExpr)
  {
    this (parent, sourceExpr, null);
  }

  public FunctionCallPStmt (PFunctionDef parent, Expr callExpr,
                            Expr _returnVar)
  {
    super (parent, callExpr);
    if (isCILFunctionCall (callExpr))
      callIndex = CILFunctionCallOp.getCallIndex (callExpr);
    else
      callIndex = -1;
    returnVar = _returnVar;
  }

  public String getFunctionName ()
  {
    if (sourceBlock != null)
      return sourceBlock.getFunctionCallName ();

    if (sourceExpr != null)
      return CILFunctionCallOp.getName (sourceExpr);

    return null;
  }

  public Object accept (PProgramVisitor v, Object o)
  {
    return v.visitFunctionCall (this, o);
  }
  

  public String getResultVarName ()
  {
    if (sourceBlock != null)
      return sourceBlock.getFunctionCallReturnVarName ();
    
    return null;
  }

  public Expr getReturnVar ()
  {
    if (sourceExpr != null)
      return returnVar;

    return null;  
  }

  public void setReturnVar (Expr _returnVar)
  {
    returnVar = _returnVar;
  }

  public Block getArgumentsBlock ()
  {
    return sourceBlock.getFunctionCallArgs ();
  }

  public Expr getArgumentsExpr ()
  {
    return CILFunctionCallOp.getArguments (sourceExpr);  
  }

  /** 
   * Returns a CILListOp.LIST Expr beginning with the return index value,
   * followed by the argument values.
   *
   * @return 
   */
  public Expr getFullArgumentsExpr ()
  {
    List args = new LinkedList ();
    if (isCILFunctionCall (sourceExpr))
    {
      args.add (CILFunctionCallOp.getCallIndexExpr (sourceExpr));
      if (CILFunctionCallOp.hasArguments (sourceExpr))
	args.addAll (CILFunctionCallOp.getArguments (sourceExpr).args ());
      //System.err.println (getFunctionName () + " call has #args: " + args.size
      //());  
      return getParent ().getParent ().getExprFactory ().
	op (CILListOp.LIST).naryApply (args);
    }
    else if (isCCallExpr (sourceExpr))
    {
      ExprFactory efac = sourceExpr.getFactory ();
      args.add (efac.intExpr (callIndex));
      args.addAll (CCallExprOp.getArgs (sourceExpr).args ());
      return TupleOp.tuple (args);
    }
    throw new RuntimeException ("Unknown sourceExpr " + sourceExpr.op ());
  }

  public int getCallIndex ()
  {
    return callIndex;
  }

  public void setCallIndex (int _callIndex)
  {
    callIndex = _callIndex;
  }

  public void setLogues (PStmt _logues)
  {
    logues = _logues;
  }

  public PStmt getLogues ()
  {
    return logues;
  }

  /**
   * Sets callee to {@code v}.
   */
  public void setFunctionDef (PFunctionDef v)
  {
    functionDef = v;
  }

  public void setCallee (PFunctionDef callee)
  {
    functionDef = callee;
  }

  public PFunctionDef getFunctionDef ()
  {
    return functionDef;
  }

  public PFunctionDef getCallee ()
  {
    return functionDef;
  }

  public void printMe (PrintWriter out)
  {
    out.println ("function-call " + sourceExpr);
  }

  public int getLineNum ()
  {
    Expr src = getSourceExpr ();
    if (src == null) return super.getLineNum ();
    if (src.arity () != 5)
      {
	System.err.println ("Strange function call without a line number");
	System.err.println ("Srouce expr: " + src);
	return super.getLineNum ();
      }
    
    
    return RationalOp.getN (src.arg (4));
  }

}

