package edu.toronto.cs.yasm.pprogram;

import edu.toronto.cs.expr.*;

import edu.toronto.cs.cparser.block.*;

import java.io.PrintWriter;


/**
 * IfPStmt.java
 *
 * Note that thenGoto.dest is initialized to thenStmt, PProgramCompiler uses
 * thenGoto.dest, PredicateRefiner uses thenStmt, and similarly for else.
 *
 * In other words, it looks like only one of thenStmt and thenGoto is actually
 * needed, as they're redundant.
 *
 * Note that themStmt and elseStmt are not next-linked in to the main
 * PProgram; they're only referred to by their containing IfPStmt.
 *
 * I think Arie explained to me why at some point we needed both thenGoto and
 * thenStmt, but I can't recall why now.
 * 
 * Nothing seems to use exitGoto, although its dest is set in PProgramPatcher
 *
 * Created: Fri Jun 25 14:25:25 2004
 *
 * @author <a href="mailto:kelvin@tallinn.cs">Kelvin Ku</a>
 * @version 1.0
 */
public class IfPStmt extends PStmt
{
  GotoPStmt thenGoto;
  GotoPStmt exitGoto;
  GotoPStmt elseGoto;

  PStmt thenStmt;
  PStmt elseStmt;

  PCond cond;

  public IfPStmt (PFunctionDef parent, Block sourceBlock, PCond _cond,
                  PStmt _thenStmt, PStmt _elseStmt)
  {
    super (parent, sourceBlock);

    cond = _cond;
    thenStmt = _thenStmt;

    thenGoto = new GotoPStmt (thenStmt);
    elseStmt = _elseStmt;
    elseGoto = new GotoPStmt (elseStmt);
    exitGoto = new GotoPStmt ();
  }

  public IfPStmt (PFunctionDef parent, Expr sourceExpr, PCond _cond,
                  PStmt _thenStmt, PStmt _elseStmt)
  {
    super (parent, sourceExpr);

    cond = _cond;
    thenStmt = _thenStmt;

    thenGoto = new GotoPStmt (thenStmt);
    elseStmt = _elseStmt;
    elseGoto = new GotoPStmt (elseStmt);
    exitGoto = new GotoPStmt ();
  }

  public Object accept (PProgramVisitor v, Object o)
  {
    return v.visitIf (this, o);
  }
  

  public PStmt getThenStmt ()
  {
    return thenStmt;
  }

  public void setThenStmt (PStmt _thenStmt)
  {
    thenStmt = _thenStmt;
  }

  public PStmt getElseStmt ()
  {
    return elseStmt;
  }

  public void setElseStmt (PStmt _elseStmt)
  {
    elseStmt = _elseStmt;
  }

  public PCond getCond ()
  {
    return cond;
  }

  public void setCond (PCond _cond)
  {
    cond = _cond;
  }

  public GotoPStmt getThenGoto ()
  {
    return thenGoto;
  }

  public GotoPStmt getElseGoto ()
  {
    return elseGoto;
  }

  public GotoPStmt getExitGoto ()
  {
    return exitGoto;
  }

  // Assumes branch targets have been patched and labelled
  public void printMe (PrintWriter out)
  {
    out.print (getId () + "/" + getLabel () + ": ");
    out.print ("if (" + cond + ") goto " + thenGoto.getTargetLabel () +
      " else goto " + elseGoto.getTargetLabel ());

    if (sourceBlock != null)
      out.println (" (" + sourceBlock.getLineNum () + ")");
    else
      out.println ();
  }

  public int getLineNum ()
  {
    Expr src = getSourceExpr ();
    if (src == null) return super.getLineNum ();
    
    return 
      RationalOp.getN (src.arg (3));
  }
  
}
