package edu.toronto.cs.yasm.pprogram;

import edu.toronto.cs.modelchecker.StatePresenter;
import edu.toronto.cs.yasm.reachability.ReachabilityStructure;


/**
 * An interface for a compiler that transforms a {@link PProgram} into
 * a {@link ReachabilityStructure} for reachability analysis.
 *
 * @author <a href="mailto:arie@cs.toronto.edu">Arie Gurfinkel</a>
 * @version 1.0
 */
public interface PProgramReachabilityCompiler
{
  /**
   * Sets the program to be compiled.
   *
   * @param p a <code>PProgram</code> to be compiled.
   */
  public void setPProgram (PProgram p);
  
  /**
   * Sets the maximum decision diagram variables to be allocated for
   * the life of this compiler. <b>Note:</b> the effect of this call
   * depends on the underlying decision diagram package used. Some
   * packages require that all variables are allocated in advace, some
   * do not.
   *
   * @param v maximum number of DD nodes to allocate.
   */
  public void setMaxDDVars (int v);
  /**
   * Returns a {@link StatePresenter} that converts decision diagrams
   * constructed by this compiler back into expression form.
   *
   * @return a <code>StatePresenter</code> value
   */
  public StatePresenter getStatePresenter ();
  
  /**
   * Returns a compiled {@link ReachabilityStructure}. This is the
   * main method of the compiler interface.
   *
   * @return a <code>ReachabilityStructure</code> representing the
   * compiled {@code PProgram}.
   */
  public ReachabilityStructure compile ();
  
}
