package edu.toronto.cs.yasm.pprogram;

import edu.toronto.cs.expr.*;

import java.util.List;
import java.util.ArrayList;

/** 
 * A class to represent predicates. A predicate include an Expr which
 * represents the name of the predicate (ie. "x >= 0"), as well as
 * metadata. Currently, the metadata we track is the source of a
 * predicate, which is either a conditional, or a wp of an assignment
 * over another predicate.
 *
 * PredicateTable.Predicate grabbed the name "Predicate" first, so
 * this class is called "Pred."
 */
public class Pred
{
  Expr name;
  PredicateSource source;
  List<String> preconditionFor;

  void init (Expr e, PredicateSource p) {
    name = e;
    source = p;
    preconditionFor = new ArrayList<String> ();
  }

  public Pred (Expr e, PredicateSource p) {
    init (e, p);
  }

  public Pred (Expr e, PredicateSource p, String label) {
    init (e, p);
    preconditionFor.add (label);
  }

  public Expr getName () 
  {
    return name;
  }

  public PredicateSource getSource () 
  {
    return source;
  }

  public boolean isPrecondition ()
  {
    return !preconditionFor.isEmpty ();
  }

  /**
   * Return a list of labels for whose statements this {@code Pred} is
   * thought to be a precondition.
   *
   */
  public List<String> preconditionLabels ()
  {
    return preconditionFor;
  }

  public String toString ()
  {
    return name.toString () + source.toString ();
  }

  /** 
   * Convenience function: given a list of @code{Pred}s, convert it 
   * into a list of @code{Expr}s
   *
   * @param l a @code{List<Pred>} value
   */
  public static List<Expr> getNames (List<Pred> l)
  {
    List<Expr> r = new ArrayList<Expr> ();
    for (Pred p : l) {
      r.add (p.getName ());
    }
    return r;
  }
}
