package edu.toronto.cs.yasm.pprogram;

import edu.toronto.cs.expr.*;
import java.io.PrintWriter;
import java.util.*;

/** 
 * SwitchBranchPStmt.java 
 *
 * A switch statement: a multi-way branch statement with each branch defined
 * by a PSwitchCase.
 *
 * Like a ConditionalBranchPStmt, a SwitchBranchPStmt does not contain the
 * bodies of its cases, but merely contains a list of PSwitchCases which point
 * at the case statements.
 *
 * Note that {@code dests} is a list of CFG successors, but does not indicate
 * which case corresponds to which dest. Use {@code cases} to get the exact
 * (case, dest) pairs.
 * 
 * @author Kelvin Ku (kelvin@cs.toronto.edu)
 * @version 
 */
public class SwitchBranchPStmt extends PStmt
{
  List<PSwitchCase> cases;

  public SwitchBranchPStmt
    (PFunctionDef parent, Expr sourceExpr, List<PSwitchCase> _cases)
  {
    super (parent, sourceExpr);
    cases = _cases;
  }

  public Expr cond ()
  {
    return CSwitchExprOp.switchCond (sourceExpr).arg (0);
  }

  public List<PSwitchCase> cases ()
  {
    return cases;
  }

  public void printMe (PrintWriter out)
  {
    out.println (getId () + "/" + getLabel () + ": ");
    out.println ("switch (" + cond () + ")");

    for (PSwitchCase c : cases)
    {
      out.print ("(");
      c.printMe (out);
      out.println (")");
    }
  }


}
