package edu.toronto.cs.yasm.wp;

import edu.toronto.cs.expr.*;
import edu.toronto.cs.tp.SymbolTable;
import java.util.*;

/** 
 * {@code WPComputer} for dependant (sequential) composition.
 * 
 * @author Kelvin Ku (kelvin@cs.toronto.edu)
 * @version 1.0
 */
public class DCWPComputer extends BaseWPComputer implements WPComputer
{
  // -- one WPComputer per assignment, in reverse order
  LinkedList wpComputers;

  /**
   * Creates a new <code>DCWPComputer</code> for a basic block
   * (a.k.a. sequential composition) of statements in an expression
   * list.
   *
   * @param stmtList a basic block
   */
  public DCWPComputer (Expr stmtList, SymbolTable st)
  {
    assert (stmtList.op () == CILListOp.SLIST) :
      "Invalid Expr: " + stmtList;

    wpComputers = new LinkedList ();

    // -- create WP computers list in reverse order
    for (Iterator it = stmtList.args ().iterator (); it.hasNext ();)
      wpComputers.addFirst (WPComputerFactory.wp ((Expr) it.next (), st));
  }

  public Expr computeWP (Expr e)
  {
    Expr post = e;

    for (Iterator it = wpComputers.iterator (); it.hasNext ();)
      post = ((WPComputer) it.next ()).computeWP (post);

    return post;
  }

}

